#ifndef _H2_net_H_
#define _H2_net_H_

#include <stdio.h>
#include <stdarg.h>
#include "topology.h"

// we separate the hyperbolic and the SOM stuff in two base classes:

class h2_node: public node {  friend class h2_net;
   // the base class to make a H2-lattice-node:
public:
   float fX,fY;       // node location in projected H2 (Poincare-Model)
   int iRadialDist;   // radial distance from center (in lattice units)
   h2_node(int iIdx, int iNumOrder): node(iIdx,iNumOrder) {};
   ~h2_node(void) { free(ppNb); }
};

class h2_net: public topology {
   int iMaxRadius;
   float fH2InvEdgeLength;
   h2_node **ppNode;
 public:
   float *get_pos(int i) { return &(ppNode[i]->fX); } 
   float dist0(float*,int);      // distance between point and a node
   float dist1(int,int);         // distance between point and a node
   float dist(float*,float*);    // distance between two points
   void translate(float*, float*); // translation
   void rotate(float*, float*);    // rotation
   int create_search_range(int**); // create search range array & ret dim 
   h2_net(int, int); 
   virtual ~h2_net(void);
};

#endif
