/******************************************************************************
 *
 * neo0.h  -  headers for neo0.c
 *
 * Copyright Helge Ritter, 1994
 *
 * $Source$
 * $Log$
 * Revision 1.18  2005/06/09 14:47:15  rhaschke
 * completely reworked NeoModifyContainer and associated utility routines
 * to shuffle pins and wires of the container
 *
 * Revision 1.9  2000/06/24 20:48:22  helge
 * (bug correction) array subwindows with the special wildcard
 *    index "$#" were not properly positioned into their host window frame.
 * (change) nst_strdup() now in nst0.c
 *
 * Revision 1.8  1999/03/09 13:38:03  helge
 * new _init method:  "_init" is now a reserved name to designate
 *   initialization units that can accomodate initialization operations:
 *   after each instantiation, copy or modification of a unit u, Neo will
 *   now traverse all its (u's) subunits recursively, executing those units
 *   that bear the reserved name "_init" in the order of their occurrence.
 *
 * Revision 1.7  1999/02/23 10:46:18  helge
 * remaining changes for Neo cooperation with array_of unit
 *
 * Revision 1.6  1998/11/26 17:17:48  helge
 * Containers: interactive insertion/deletion of single pins from a
 * composite field could screw up connections of wires connected to
 * pins behind the insertion/deletion point.  Now corrected.
 *
 * Revision 1.5  1997/12/21 00:07:35  helge
 * Mainly changes to support major new feature of ``class containers'':
 * C++ classes can now be wrapped as NST units such
 * that a class instance appears as a ``class container'' with subunits
 * representing the methods.
 * Necessary wrapper code is generated with new tool mku++.
 * Similar procedure for C ``classes'' supported by new tool mku+
 *
 * Revision 1.4  1997/07/31 09:15:17  jan
 * Missing prototype added.
 *
 * Revision 1.3  1997/04/02 12:46:46  helge
 * several minor corrections, among them: better handling of operator scopes,
 * ~-expansion in .neofolders-file, unloading of dynamic folders now works,
 * save gz file now safeguarded against specification of non-existent dir.
 *
 * Revision 1.2  1997/03/13  09:58:58  jan
 * Folder neuron extracted. Dependency cleanup No. 1: no more nst_defs.h.
 *
 * Revision 1.10  1997/01/31 20:02:17  helge
 * (minor changes or only touched)
 *
 * Revision 1.9  1997/01/14  14:34:32  helge
 * changes 135-139 bugs 131,132, numerous further small changes to
 * take care of (almost) all warnings under -Wall -O2
 * !!! Some Neo-structs have changed, trust the Makefile or  !!!
 * !!! recompile EVERYTHING !!!
 *
 * Revision 1.8  1996/08/07  14:19:35  helge
 * New NST-Version 4.8 / Neo-Version 2.8  accumulating major changes
 *
 * Revision 1.7  1995/08/02  16:44:40  helge
 * Load/Save can now use gzipped files.
 * Src code generation now works also for new pin types.
 *
 * Revision 1.4  1995/07/16 20:58:37  helge
 * added support for sending units to remote host(s) [in neo8.c]
 * better interface for container_unit
 *
 * Revision 1.3  1995/05/30  12:05:34  mkrause
 * -gensrc added
 *
 * Revision 1.2  1995/04/18  21:31:15  helge
 * many bug fixes. Mouse unit has changed. Major improvements are:
 * - man pages can now be fetched from within neo
 * - each neo program can have an associated .HLP file for instructions
 * - initial version of a "patch mechanism" for changing dialog windows
 *   without losing compatibility with earlier versions
 * - new units include: cam_head,  box_filter, array_of
 *   [the latter very alpha]
 * - values for a user-defined root dialog window can be set via
 *   command line arg-value pairs of the form @0=value @1=value ...
 *   This allows to parametrize neo programs from the command line.
 * - a collection of demos (currently in /homes/helge/nst/neosrc/tutorial
 *   is now available.
 *
 * Revision 1.1  1995/03/06  15:17:16  helge
 * source files for neo kernel.
 *
 *
 *****************************************************************************/
#ifndef __NEO0_H
#define __NEO0_H

extern unitptr guStrictCopyOrig; /* when making a copy: imitate window
                                    placement of this unit, if defined.  
                                    (i.e., disable autoplace) */ 
  
extern char* NeoMakeName(char *pcName, unitptr uIcon);
extern char* NeoMakeName1(char *pcName, unitptr uIcon);
extern char* NeoMakeName2(char *pcName, char *pcRemoveSet, 
                          char *pcCapSet, unitptr uIcon);
extern char* NeoMakeName3(char *pcName, char *pcRemoveSet, 
                          char *pcCapSet, unitptr uIcon);

extern void NeoSetDialogWindow (unitptr uDialogWindow);

extern int copy_input_window_items (int iFirst, int iNum, unitptr uDest, unitptr uSrc);
extern char input_win_itemtype_at (int iPos, unitptr u);
extern int num_input_entries_of_fmt (char *pcFmt);
extern int num_input_entries_of (unitptr u);
extern char find_input_win_item_at (int iPos, unitptr u, int* piTxtIndex, int* piFloatIndex);
extern int findTextFieldOfNumVal (int iNumPin, char *pcFmt);
extern void simplifyNumVals (unitptr uWdw);

extern int isTypeCompatiblePair (field *pF1, field *pF2);
extern int isFullyTypeCompatiblePair (field *pF1, field *pF2);

extern unitptr getIconFromOrig(unitptr uOrig, unitptr uSearchRoot);

extern int RemoveUnit(unitptr u);
extern int exec_queue(unitptr u);
extern int neo_adapt_unit(unitptr u);
extern int neo_exec_unit(unitptr u);

extern int HasSubunits (unitptr u);
extern unitptr SubunitAtPos (int iPos, unitptr uHost);
extern int CanReplace (unitptr u, unitptr v);
extern int NeoWireConnectedToIcon (unitptr uWire, unitptr uIcon);
extern int ScalarPinsUnconnected(field *pF, int iFirst, int iNum);
extern void SwapValuesOfCorrespondingPinsIfFreeOnArg1 (unitptr u, unitptr v);
extern void WriteWire (unitptr uWire);

extern int joinfields_ff(field *pF1, field *pF2);
extern int joinpins_ff(field *pF1, field *pF2, int j1, int j2);

extern pin* NeoAddPin (pin *pPinType, int iDim, int iPos, field *pField, unitptr uIcon);
extern int  NeoRemovePin(int iPos, field *pField, unitptr uIcon);
extern void NeoModifyPin(char cType, int iDim, int iPos, 
                         int iField, int isOut, unitptr uIcon);
extern void NeoSwapAdjacentPinRegionsOfField (unitptr uIcon, int iField, int iIsOut, int iFirst, int iDimFirst, int iSecond, int iDimSecond);
extern int NeoMovePinRegions (unitptr uIcon, int iIsOut, int iSrcField, int iSrcPin, int iDim, int iDstField, int iDstPin, int iCreate);

extern int NeoAddField (int iPos, char *pcFmt, int isOut, unitptr uIcon);
extern int NeoRemoveField(int iPos, int isOut, unitptr uIcon);
extern int NeoSwapFields (unitptr uIcon, int iField1, int iField2, int iIsOut);

extern char* NeoSubstituteArrayIndexToken(unitptr uWdw);
extern void NeoSwapParametersForArrayOf(unitptr uFrom, unitptr uTo);
extern unitptr neo_array_of(int iNumDim, int *piDim, unitptr *puArray, unitptr uHost);

#endif
