/******************************************************************************
 *
 * neo2.h  -  icon units 
 *
 * Copyright Helge Ritter, 1994
 *
 * $Source$
 * $Log$
 * Revision 1.21  2006/11/22 07:57:28  rhaschke
 * extended input field length for %f,g,i,j fields
 *
 * Revision 1.20  2005/09/29 10:34:02  rhaschke
 * several bug fixes to new event handling scheme
 *
 * Revision 1.19  2005/09/02 13:44:09  rhaschke
 * merge with Helge's NST7 version: simplified event loop, smaller neo-files
 * major cleanup of compiler switches, declarations, etc.
 *
 * Revision 1.13  2000/05/15 10:21:33  helge
 * (bug correction) NeoCmdStep(): 2nd "Step" for a unit with window(s)
 * in autostart mode had unwanted side effect to disable autostart mode.
 *
 * Revision 1.12  2000/02/01 16:01:55  helge
 * bug correction: instantiation of by-reference icon could occur with
 *    internal subunit-order screwed up.
 * new function NeoCircuitIsModifyable().
 *
 * Revision 1.10  2000/01/14 16:14:32  helge
 * adjustments to new versioning scheme for loadable units/folders:
 * 1. ShiftLeft while dragging loadable icon allows to choose older versions
 * 2. new command line options:
 *    "-exactversion" to enforce exact version matches
 *                    (instead of newest minor version)
 *    "-ignoredata"   to ignore data part of circuit when loading
 * 3. exactversion, ignoredata and trace loading (-TL) options now also
 *    settable in "Load" command window
 * 4. added startup message about load paths used to locate libraries and files
 *
 * Revision 1.8  1999/03/09 13:38:05  helge
 * new _init method:  "_init" is now a reserved name to designate
 *   initialization units that can accomodate initialization operations:
 *   after each instantiation, copy or modification of a unit u, Neo will
 *   now traverse all its (u's) subunits recursively, executing those units
 *   that bear the reserved name "_init" in the order of their occurrence.
 *
 * Revision 1.7  1999/02/14 17:49:46  helge
 *  - neo_main.c icons can now become activated (executed) when the
 *    cursor moves into a window that has been associated with the
 *    icon before (specification occurs in Define dialog).
 *  - neo.h neo1.c neo2.c neo3.c neo4.c neo5.c neo8.c neo9.c neo10.c
 *    neo_main.c : changes to allow loading of icon units from loadable
 *    folders "by reference": such icons cannot be changed internally
 *    and they will always stay in sync with their definition in
 *    the folder.
 *  - neo_main: changed color initialization to newer rgb:rr/gg/bb
 *              format to avoid problems with the initial '#'-symbol (which
 *              unfortunately is already reserved as a comment symbol).
 *  - optimization of instantiation of loadable icons from folders:
 *    so far, a loadable icon was made with its default parameters,
 *    and then remade when changes were requested. Now the query for
 *    changes is made earlier (in NeoReadHeaderIcon()) so that changes
 *    can already become effective during the first instantiating
 *    process and no further remake cycle is necessary.
 * - minor cleanup: EditWire() is now NeoEditWire() (affects
 *   neo?.c, neo3.h, neo_stdr.c)
 *
 * Revision 1.4  1997/05/20 07:52:44  helge
 *     neo2.c:  Move command sometimes incorrectly refused move.
 *     ESC from loops was not always properly reset. New routines
 *     for feedback (neo2.h)
 *     neo8.c: FIND_HOST for proper window reference initialization
 *     now built into load_icon() routine.
 *     neo_main.c: minor changes.
 *
 * Revision 1.1  1995/03/06  15:17:23  helge
 * source files for neo kernel.
 *
 *****************************************************************************/
#ifndef __NEO2_H
#define __NEO2_H

#include <setjmp.h>

#define LOADABLE_UNIT "circuits"   /* additional path element needed
                                      for new loadable folder structure */
#define NST_HIDEWID 30

/*********************** exports from neo2-0.c *******************************/

extern char gacNeoHome[NEOMAXNAMELEN];

/* icon unit representing currently selected icon */
extern unitptr guSelectedIcon; 
extern jmp_buf gpJmpLoadIconError;
extern int giJmpLoadIconError;
extern int giPrefSnap;
extern unitptr guRootIcon;

extern void NeoMemUsage (int iLine);

extern void NeoEndCall (unitptr uArg);
extern void NeoSetEndCall (void (*pFun)(unitptr));

extern int NeoIsNamedIcon (unitptr u);

extern unitptr NeoMakeContainer (unitptr uHost);
extern int isIconUnit (unitptr u);
extern int isContainer0 (unitptr u);
extern int isContainer1 (unitptr u);
extern int isContainer (unitptr u);
extern int NeoIsContainerOrThreadBox (unitptr u);

extern int NumSubiconsOf (unitptr uIcon);
extern int NumSubIcons (unitptr uIcon);
extern int NeoIconExportsMethods (unitptr uIcon);
extern void NeoFindWindowHosts (unitptr uIcon);
extern int NeoExecInitMethods (unitptr uIcon);
extern void NeoExecInitMethodsForOrig (unitptr uOrig);
extern void NeoReportAnyInitializationErrors (unitptr uIcon);
extern unitptr NeoFirstSelectedIcon (unitptr uHostIcon);
extern void NeoSelectIcon (unitptr uIcon);
extern int toggleIconSelectState (unitptr uIcon);
extern int NeoSelectIconsInRegion (int iXll, int iYll, 
                                   int iXur, int iYur, unitptr uHost);
extern int  deselectSubIcons (int i, unitptr uHost, int iDoRedraw);
extern void NeoDeselectAllSubicons (unitptr uHost);
extern void NeoDeselectSubicons (unitptr uHost);
extern void NeoDeselectIcon (unitptr uIcon);
extern int  setIconSelected (unitptr uIcon, int iToggle, unitptr uHost);
extern void NeoSelectNextIcon (unitptr uHost);
extern void NeoSelectPrevIcon (unitptr uHost);
extern int  NeoNumSelectedIcons (unitptr uHost);
extern int  isSelected (unitptr u);
extern void snapIcon (unitptr u);

extern char** ppcGetIconSourceBuf1 (unitptr uIcon);
extern char** ppcGetIconSourceBuf2 (unitptr uIcon);

extern int isVirtualIcon (unitptr uIcon);
extern void markOperatorsAndOperands (unitptr uHost);
extern void RepositionLastIcon (unitptr uHost);

extern int NeoChangeWiresToNewUnit (unitptr uIcon,unitptr uOrig, unitptr uNewIcon,unitptr uNewOrig);
extern field* findCopyOfField (field *pF,unitptr uCopy);
extern void NeoMakeInterfaceDescInWdw0 (unitptr uVoid, unitptr uWdw);
extern int getNumIconsInQueue(unitptr u);

/*********************** exports from neo2-1.c *******************************/

extern unitptr guSourceIconForCopy ;
extern void NeoSetDelayTime (float fDelay);
extern int NeoShiftKeyPressed (void);
extern int NeoIsInBottomMenuArea (int iMy);
extern int NeoBottomMenuActive (void);
extern void NeoEnableBottomMenu (void);
extern void NeoSetPinInfoMode (int iMode);
extern int NeoSelectsIcon (int iX,int iY );
extern unitptr NeoSelectsBoxField (float fX,float fY,field** ppField, unitptr uIcon);
extern int NeoCloseIcon (void);
extern void NeoOpenSelectedIcon (void);
extern void NeoParametersForSelectedIcon (void);
extern void NeoModifySelectedIcon (void);
extern void NeoExecuteSelectedIcon (void);
extern int NeoArrangeIcons (unitptr uIcon);
extern void NeoRedrawBottomMenu (void);
extern void NeoDisableBottomMenu (void);
extern void NeoOpenCircuit (unitptr uIcon);
extern void NeoDisplayError (unitptr uErrorIcon);
extern int adapt_icon_unit (unitptr u);
extern int remove_icon_unit (unitptr u);
extern void neo_query_pointer (NstIconWdw_t* pNstIconWdw, 
                               int *piX, int *piY, int *piButtons);

/*********************** exports from neo2-2.c *******************************/

extern int giNewFormat ;
extern int NeoDialogToCreateLoadableIcon (unitptr uIcon);
extern unitptr icon_unit_fp (FILE *fp, int iXll, int iYll, char *pcFun, 
                             int *piCancelled, unitptr uOrig, unitptr uHost);
extern char* NeoGetIconOrigin (unitptr uIcon);
extern void  NeoSetIconOrigin(char *pcFun, char *pcFolder, char *pcFolderVersion, 
                              char *pcVersion, unitptr uIcon);

extern char* folder_path_and_version_for_loadable_icon (
   char *pcIconName, char *pcFolder, char *pcFolderVersion);

extern void NeoAddVersionCmd(unitptr uIcon);
extern int FolderVersionsOk(char *pcName, char *pcFolder, 
                            char *pcRequestedVersion, char *pcAvailableVersion);
extern void CorrectVersionInPopupCmd(int iMajor, int iMinor, unitptr uNew);

extern unitptr icon_unit (int iXll, int iYll, char *pcFun, 
                          char *pcFolder, char *pcVersion, 
                          unitptr uOrig, unitptr uHost);

/*********************** exports from neo2-3.c *******************************/

extern void NeoSetSRCLINEPointer(void);
extern int  SetDontAddSrcLine (int iNew);
extern int  NeoInterfaceOfLoadedUnit (int *piNumInp, char ***pppcInp, 
                                     int *piNumOut, char ***pppcOut);
extern void NeoReplaceIcon (unitptr uFrom, unitptr uTo);
extern void NeoMakeCircuitUnmodifyable (int iYes);
extern int  NeoCircuitIsModifyable (unitptr uHost);
extern int  NeoCircuitIsNotModifyable (void);

extern void NeoEnableInstallDialog (void);
extern void NeoDisableInstallDialog (void);
extern int  NeoInstallDialogEnabled (void);

extern void neo_request_retry (void);

/*********************** exports from neo2-4.c *******************************/

extern void NeoRedrawAfterIconMove (unitptr uMoved, int iXllOld, int iYllOld, 
                                    NeoRect_t *pBoundingBox, NstIconWdw_t *pIconWdw);
extern void NeoRedrawArea (int iXll, int iYll, int iXur, int iYur);
extern int ctrl_icon_unit (int mode, unitptr u);
extern int NeoRemoveIconWithWires (unitptr uIcon);

/*********************** exports from neo2-6.c *******************************/

extern int NeoIconHasUserWdw (unitptr uIcon);
extern void NeoMoveUserWdws (unitptr uSrcIcon, unitptr uDestIcon);
extern void NeoCopyUserWdws (unitptr uSrcIcon, unitptr uDestIcon);
extern void NeoClearUserWdws (unitptr uSrcIcon);
extern void NeoClearIconBox (unitptr uIconBox, int iAll);
extern void NeoCheckAndUpdateSubIcons (unitptr uIcon);
extern char* NeoNameIcon (char *pcNewName, unitptr uIcon);
extern void NeoNameIconAtPos (int iMx, int iMy);
extern void NeoNameSelectedIcon (void);
extern char* NeoQueryVersion (char *pcName);
extern unitptr NeoModifyVoidUnit (unitptr uIcon);
extern unitptr NeoModifyIcon (unitptr uIcon);
extern unitptr NeoModifyIcon0 (unitptr uIcon, int *piWasCancelled);
extern unitptr NeoModifyIcon1 (unitptr uIcon, int *piWasCancelled);
extern int NeoWindowUsesVariables (unitptr uWdw);
extern int NeoSelectedIconsUseLocalVariables (void);
extern int NeoIconUsesVariables (unitptr uIcon);

/*********************** exports from neo2-7.c *******************************/

extern unitptr NeoCopyOfIcon (unitptr uIcon, unitptr uDest);
extern unitptr NeoStrictCopyOfIcon (unitptr uIcon, unitptr uDest);
extern int NeoMoveIcons (unitptr uSrc, unitptr uDest, char *pcReplaceVars);
extern int NeoMoveIcons0 (unitptr uSrc, unitptr uDest, char *pcReplaceVars);
extern int NeoCopySelectedIcons (unitptr uSrc, unitptr uDest);
extern void NeoToggleIgnoreSelectedIcons (void);

/*********************** exports from neo2-8.c *******************************/

extern void NeoSetHTMLViewer (const char *pcCmdTemplate);
extern int  NeoDisplayHTMLManPage (char *pcFileName);
extern int  NeoDisplayNroffManPage (char *pcFileName, char *pcTitle);
extern void NeoExecManPage(void);
extern void NeoSetManPage (unitptr uWindow, int iPreserve);
extern char* NeoFindFile(char *,char *,char *,char *);

/*********************** exports from neo2-9.c *******************************/

extern int giNeoSqueezeDy;
extern NstIconData_t gpsIconDef[];

extern int getFieldGeom (unitptr uIcon, field *pF, float *pfXll, float *pfYll, 
                         float *pfXur, float *pfYur, float *pfDy);
extern int getFieldPos (unitptr uIcon, field *pF, float *pfX, float *pfY);
extern int exec_icon_unit (unitptr u);

#endif /* __NEO2_H */
