/******************************************************************************
 *
 * neo_main.h  -  headers for neo_main.c
 *
 * Copyright Helge Ritter, 1994
 *
 * $Source$
 * $Log$
 * Revision 1.8  2005/09/29 10:34:01  rhaschke
 * several bug fixes to new event handling scheme
 *
 * Revision 1.7  2005/09/02 13:44:11  rhaschke
 * merge with Helge's NST7 version: simplified event loop, smaller neo-files
 * major cleanup of compiler switches, declarations, etc.
 *
 * Revision 1.6  2000/11/06 11:53:24  helge
 * better handling of by-reference instances:
 * now nested by-reference instances fully legalized. Required:
 * - to equip save routines with some safeguard actions against occurrence
 *   of circular references.
 * - allow access of user-defined commands also for nested by-reference units
 * - new option -closestversion to resolve reference instances with closest
 *   version instead of newest.
 * bug corrections:
 * - the by-reference flag of a loadable icon shadowed the by-reference
 *   flag of loadable icons with the same name but located in subsequent folders.
 * - proper parsing of by-reference definitions also after previous creation
 *   of dummy instances (postponed call of RedefineUserDefWdw() in this case)
 * - disabled inappropriate NeoEndOfUnit() call when dummy_icon() was made
 * - removed obsolete omit_section_endmark() call
 * - diagnostics given with -TL option could crash when they contained a '%'.
 *    [replaced fprintf(stderr,gacBuf) by fprintf(stderr,"%s",gacBuf)]
 * - loading/unloading folders with +name, -name syntax did not always
 *   find the correct folder.
 * cleanup: removed obsolete routines Load/SaveIconData().
 *
 * Revision 1.5  1997/08/26 15:45:09  jan
 * External declarations cleanup.
 *
 * Revision 1.4  1997/08/23 21:53:23  helge
 * Some changes/extensions:
 * 1. there is no longer NeoExmplPath and NeoManPath to be set in .neo
 *    or on the command line. Instead, if a folder is specified to be
 *    loaded from path/o.objectdir/folder.so, examples and manpages for
 *    this folder are now sought in path/examples, path/cat3 and path/html3.
 *    If no path is specified, or the folder is compiled, NeoHome is taken for path.
 * 2. When a circuit contains the new reserved token $LOAD_DIR in its
 *    text entry fields, this will be taken as the name of the directory
 *    from which the current circuit was loaded ("." if used in a circuit
 *    that was not loaded). This allows to reference files needed by a circuit
 *    in a way that is independent of the circuit's actual directory.
 *    For details, see log file.
 * 3. minor changes: redrawing of widget scopes, bugfix for src generation,
 *    Neo-Prefix for some routines.
 *
 * Revision 1.3  1997/04/02  12:47:07  helge
 * several minor corrections, among them: better handling of operator scopes,
 * ~-expansion in .neofolders-file, unloading of dynamic folders now works,
 * save gz file now safeguarded against specification of non-existent dir.
 *
 * Revision 1.2  1997/03/13  09:59:43  jan
 * Folder neuron extracted. Dependency cleanup No. 1: no more nst_defs.h.
 *
 * Revision 1.4  1997/01/31 20:02:50  helge
 * (minor changes or only touched)
 *
 * Revision 1.3  1997/01/14  14:35:34  helge
 * changes 135-139 bugs 131,132, numerous further small changes to
 * take care of (almost) all warnings under -Wall -O2
 * !!! Some Neo-structs have changed, trust the Makefile or  !!!
 * !!! recompile EVERYTHING !!!
 *
 * Revision 1.2  1995/08/02  16:45:06  helge
 * Load/Save can now use gzipped files.
 * Src code generation now works also for new pin types.
 *
 * Revision 1.2  1995/07/16 20:58:59  helge
 * added support for sending units to remote host(s) [in neo8.c]
 * better interface for container_unit
 *
 * Revision 1.1  1995/03/06  15:17:38  helge
 * source files for neo kernel.
 *
 *****************************************************************************/
#ifndef __NEO_MAIN_H
#define __NEO_MAIN_H

extern int giPrefSteps;     /* number of exec steps */
extern int giPrefAdapt;     /* adaptation flag */

/********************* exports from neo_main.c *************************/

extern char* gpcNeoAlternateHomePath;
extern int giPrefSaveLoadableIconsWithData;
extern int giTraceLoading;
extern int NeoNewGui (void);
extern int NeoPrefHide (void);
extern int NeoEmbedMode (void);
extern int NeoUseHTMLViewer (void);
extern void NeoQueryLoadStatus (int *piIgnoreData, 
                                int *piTraceLoading, 
                                int *piLoadWhichVersion);
extern int NeoClosestVersion(void);
extern int NeoExactVersion(void);

extern const char** gppcCommands;
extern const int    giNeoNumCommands;

extern void NeoCmdHide (int iHideEnabled, unitptr uMenue);

extern void NeoSetLabelColors (char *pcCmdLabelColor, 
                               char *pcFolderLabelColor);
extern char* NeoFolderLabelColor (void);
extern char* NeoCmdLabelColor (void);
extern int   NeoContainerLabelColor (void);
extern void  NeoRedefinePredecessor (unitptr uIcon);
extern int   NeoSetNewPrefs (void);
extern void  NeoUpdatePinInfoModeInPrefs (int iMode);

extern void (*NeoWithdrawFromApplication)(void); /* function called on exit */

extern int  NeoExitFlag (void);
extern void UpdateCircuitPrefs (void);

#include "neo_event.h"
extern void UpdateFocusObj (neo_event_t *pEv, int iIncludeCurrentFocusObj);
extern void UpdateFocusObjFromPointer (NstIconWdw_t* pNstIconWdw, int iCurObj);
extern void NeoInvalidateAsFocusObj (unitptr uObj);

/********************* exports from neo_main-3.c *************************/

extern void NeoWritePosInfoSection (FILE *fp, unitptr uHost);
extern int  NstDataFileFollows (FILE *fp, unitptr uHost);
extern void SaveData (FILE *fp, unitptr uIconToSave);

/********************* exports from neo_main-4.c *************************/

extern void NeoDefineDefaultExampleDir (void);
extern void NeoNewCircuitRoot (unitptr uOrig);
extern unitptr NeoGetRootIconOfCircuit (int iWhich);
extern void NeoLoadExampleFor (unitptr uIcon, NstFolder_t *psFolder);
extern void NeoUpdateLoadPathFor (unitptr uIcon, char *pcLoadPathOfIcon);
extern void NeoRedefineLoadPath (char *pcLoadPath);
extern int NeoLoadPathOkForSave ( unitptr uHost, char *pcFile );
extern unitptr NeoUpdateMenue (unitptr uMenue, int iNewFolders);
extern int NeoAutoStartEnabled (void);
extern char* NeoGetCircuitName (void);
extern int NeoGetCircuitOf (unitptr uIcon);
extern char* NeoGetCircuitPath (void);

extern int NeoGetCurrentCircuit (void);
extern int NeoTestMode(void);

/********************* exports from neo_main-5.c *************************/

extern void NeoClearExitFlag(void);
extern unitptr NeoCmdLoad(char **apcArg,   int iLoadMode);
extern void NeoCmdRun(int iNumSteps);
extern void NeoCmdShowInfo(char *pcTextFile, char *pcHtmlFile);
extern void NeoCmdStep(unitptr uIcon);
extern char* NeoIconTitleFromOrig(unitptr uOrig, unitptr uHostIcon);

#endif /* __NEO_MAIN_H */
