/*****************************************************************************
 *
 * nst.h  -  architecture fixup header
 *
 * Copyright Helge Ritter, 1992 1993
 *
 * $Source$
 *
 * $Log$
 * Revision 1.21  2005/10/28 07:30:37  rhaschke
 * cleanup of arch-dependent switches
 *
 * Revision 1.20  2002/07/17 10:30:13  helge
 * raised version nr to 7.9
 *
 * Revision 1.19  2002/01/15 09:32:27  helge
 * raised NST version nr from 7.7 to 7.8
 *
 * Revision 1.18  2001/09/14 14:34:57  helge
 * raised version to 7.7
 *
 * Revision 1.17  2001/07/02 11:04:11  helge
 * NST version raised to 7.6. Summary of changes:
 * - plots: now arbitrary choice of foreground/background color with %f:gC option
 *   for plot units, e.g., %0:1C yields black plot on white background, %C chooses
 *   window color as background and BLACK as foreground.
 * - plotting colors: now color 1 means foreground (which still is WHITE if
 *   background is not changed from default BLACK, otherwise, foreground will be
 *   BLACK). To choose white/black irrespective of current fore/background choice, use
 *   white = 8, black = 9. The previous values 8.. for window and widget colors
 *   have become shifted by 2 and now start at 10.
 * - use_method: now names can be up to 40 chars long
 * - prog_unit: added function sgn(x) for sign of x.
 * - vector assigments for subrange:  v[i..j] := vec_of_dim_j-i+1
 * - C-like tests of instantiation:   if (!object) { ... }, if (!ptr) { ... }
 *   [but if (object) will not work, use if (!!object) instead!]
 * - new built-in prog library "random":
 *     #import random
 *     ... use rnd_vec(..) and further functions ..
 * - plot_vec:  now offers dynamic dim input when specifying dim=0
 * - sleep_unit: now includes automatic flush of X-graphics queue, unless
 *   sleep time is specified with negative sign.
 * - input_window: now labelled buttons automatically stretch to width
 *   required to accomodate label.
 * - new ctrl call ctrl(NST_RENAMED,u) to notify units that have become renamed
 * - several (in most cases minor) bug corrections
 *
 * Revision 1.16  2001/02/13 10:03:45  helge
 * raised NST version nr to 7.5
 *
 * Revision 1.15  2001/02/05 14:37:52  helge
 * raised NST version nr to 7.4
 *
 * Revision 1.14  2000/06/24 19:13:05  helge
 * new minor version nr: now NST 7.3
 *
 * Revision 1.13  2000/02/07 17:22:17  helge
 * raised version nr from 7.1 to 7.2
 *
 * Revision 1.12  2000/01/10 19:52:17  helge
 * removed aux_malloc.h
 *
 * Revision 1.11  2000/01/10 17:07:28  helge
 * raised version to 7.1  and #defined S2DYN to allow scalar->dynfloat wires
 *
 * Revision 1.10  1999/05/31 18:14:57  helge
 * Added inob-field (inob=interface-object) in *unitptr struct to allow some
 * Neo-related extensions. Other changes mainly removal of files from standard
 * libnst.a into dynamic folders (moved into foldersrc, cf. files added there)
 *
 * Revision 1.8  1997/12/21 00:04:53  helge
 * several minor changes
 *
 * Revision 1.7  1997/10/22 16:14:08  helge
 * added def for version checking
 *
 * Revision 1.6  1997/05/06 07:22:10  jan
 * Added missing function prototypes for sun4.
 *
 * Revision 1.5  1997/04/29 20:09:52  helge
 * new include file nst9.h
 *
 * Revision 1.4  1997/04/15 13:47:31  jan
 * Added 'nst_dbug.h' to the inclusion list to get the prototype for
 * 'nst_warning'.
 *
 * Revision 1.3  1997/04/02 14:06:44  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.2  1997/03/13  10:48:20  jan
 * Dependency cleanup.
 *
 *
 *****************************************************************************/

#ifndef __NST_H
#define __NST_H

/*****************************************************************************

  Increment major version nr whenever a change to a NST header has been
  made that is sure to require recompilation of all used NST units.
  Increment minor version nr for less dramatic changes that, however,
  might be worth a warning.

*****************************************************************************/

#define NST_HEADER_VERSION "7.9 17/07/02 HR"

/***************************************************************************/

#define S2DYN /* enable code to allow connecting scalar vect -> dyn float vect */

#include "nst0.h"
#include "nst1.h"
#include "nst2.h"
#include "nst3.h"
#include "nst4.h"
#include "nst5.h"
#include "nst6.h"
#include "nst7.h"
#include "nst8.h"
#include "nst9.h"
#include "nst10.h"
#include "nst_dbug.h"

#endif  /* header control */

