/*****************************************************************************
 *
 * nst0.h  -  NST kernel layer 0: General definitions
 *
 * Copyright Helge Ritter, 1994
 *
 * $Source$
 *
 * $Log$
 * Revision 1.36  2006-12-22 11:31:44  rhaschke
 * added const char* here and there
 *
 * Revision 1.35  2005/11/11 08:10:23  rhaschke
 * added nst_usleep() declaration
 *
 * Revision 1.34  2005/10/31 14:40:34  rhaschke
 * cleanup of arch-dependent switches
 *
 * Revision 1.33  2005/10/14 13:08:13  rhaschke
 * added ctrl-modes
 * NST_W_NEWSIZ   - (user-triggered) resize notify
 * NST_W_ORIGSIZ  - restore original size
 *
 * Revision 1.32  2005/10/14 11:09:01  rhaschke
 * added nst_strrncmp
 *
 * Revision 1.31  2005/09/18 21:12:31  rhaschke
 * made CALLOC efence-safe
 *
 * Revision 1.30  2005/09/02 13:57:33  rhaschke
 * cleanup in declarations
 * nst_[scrn,X11,prog]: adjustments and additions for NST7's neosrc
 *
 * Revision 1.29  2005/08/08 09:50:30  rhaschke
 * replaced deprecated "values.h" by "limits.h" and "floats.h"
 * minor changes
 *
 * Revision 1.28  2005/06/10 07:45:53  rhaschke
 * cleanup to avoid (some) compiler warnings
 *
 * Revision 1.27  2002/07/17 10:41:06  helge
 * added new ctrl mode NST_THREAD_CANCEL to cancel running threads
 *
 * Revision 1.26  2002/03/15 15:51:02  helge
 * added new ctrl mode NST_W_UNPARENT for new NST window handling scheme
 *
 * Revision 1.25  2002/03/04 15:44:28  rhaschke
 * added nst_tokenize_str to import list
 *
 * Revision 1.24  2001/10/30 15:43:47  rhaschke
 * added control mode NST_W_SETWIDGETS to allow widget setting from within neo/nst
 *
 * Revision 1.23  2001/09/14 14:30:42  helge
 * (bug correction)  nst_strmatch() ignored escaping of special
 * characters []*^ and ?. New function nst_strmatch1() allows trailing
 * non-matching chars.
 * new ctrl modes NST_W_{LOAD|SAVE}_POSINFO to support load/save of
 * window positioning info also in NEO circuit files.
 *
 * Revision 1.22  2001/07/02 11:05:51  helge
 * ppp.c : !!! update this for proper creation of neo !!!
 * [now no longer any output to stderr (annoying in some cases)]
 *
 * Revision 1.21  2000/10/16 15:33:02  helge
 * (minor change) moved declaration of gfNstNaN from nst5 to nst0
 *
 * Revision 1.20  2000/07/03 16:40:58  helge
 * moved nst_find_file_version() from neo18.c to here
 *
 * Revision 1.19  2000/06/24 19:15:38  helge
 * added routines nst_extrac_name() and nst_dlcall()
 *
 * Revision 1.18  2000/04/07 14:57:21  helge
 * added definition for new return value NST_EXCEPTION
 *
 * Revision 1.17  2000/01/10 17:04:36  helge
 * new function  nst_suffix_and_version() for file name version nr parsing
 * nst_fscanf-functions can now have NULL data ptr to allow skipping of data
 *
 * Revision 1.16  1999/12/17 06:39:47  helge
 * inserted cast in REALLOC macro to avoid warnings on Solaris
 *
 * Revision 1.15  1999/12/13 08:06:54  helge
 * libc.so.5:  realloc(p,0) returns non-NULL pointer (which is not what its
 *    manpage promises!!). Likewise, calloc and malloc don't return NULL on
 *    size=0. Redefined CALLOC() and REALLOC() macros to cover size=0 case
 *    properly. Defined also MALLOC() to avoid similar nuisance with malloc(0).
 *
 * Revision 1.14  1999/09/27 15:00:39  helge
 * !!! NstInt_t changed from "short int" to "int" to permit
 *     efficient loadlib definitions for int and int* arguments.
 *
 * Revision 1.13  1999/06/16 15:11:31  helge
 * added nst_dlopen_path() to return file path chosen by most recent nst_dlopen()
 *
 * Revision 1.12  1997/12/21 15:50:43  helge
 * minor adjustments
 *
 * Revision 1.11  1997/08/30 19:10:10  helge
 * has now the file-io routines of former nst_util.c
 *
 * Revision 1.10  1997/08/26 15:43:38  jan
 * Missing external declaration added.
 *
 * Revision 1.9  1997/07/11  11:12:54  helge
 * some bugs removed, some things improved. See file "log" for details.
 *
 * Revision 1.8  1997/04/29 15:11:05  helge
 * changes necessary with introduction of user-specified data types.
 *
 * Revision 1.7  1997/04/02 14:06:53  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.6  1997/04/02 06:59:40  helge
 * intermediate check-in
 *
 * Revision 1.5  1997/03/25  12:09:03  jan
 * Removed 'nst_warning' declaration which belongs in 'nst_dbug.h'.
 *
 * Revision 1.4  1997/03/21 12:46:03  jan
 * Declaration fixes.
 *
 * Revision 1.3  1997/03/18 13:51:05  jan
 * Added missing 'extern' declarations.
 *
 * Revision 1.2  1997/03/13 10:48:24  jan
 * Dependency cleanup.
 *
 *
 *****************************************************************************/

#ifndef __NST0_H
#define __NST0_H

#include <stdio.h> 
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include "nst_types.h"

extern int (*nst_usrbreak)(void);  /* if !NULL, this routine is
                                      called in loops to check for 
                                      break requests. */
extern void NstFree(void*,char*,int);  /* diagnostic free routine */
extern char *NstStrDup(char*);
extern char *nst_str_subst(int, int, char*, char*, char*);
extern void *gpCheckAdr;    /* global address to be used for diagnostics */

#define NST_STRBLKSIZ 20   /* NST strings grow/shrink in these size steps */
#define NST_NO_WINDOW -999 /* index to indicate undefined window */

#define NSTFREE(P) {free(P);P=NULL;}  /* NstFree(P,__FILE__,__LINE__) */

#define NSTKERNEL_LAYER0

/* for debugging: */

#ifdef NST_DEBUG
#define dprint(A) fprintf(stderr,"line %d (%s): %s=%d\n",\
        __LINE__,__FILE__,#A,(A));

#define gprint(A) fprintf(stderr,"line %d (%s): %s=%g\n",\
        __LINE__,__FILE__,#A,(A));

#define tprint(A) fprintf(stderr,"line %d (%s): %s=%s\n",\
        __LINE__,__FILE__,#A,(A));

#define cprint(A) fprintf(stderr,"line %d (%s): %s\n",\
        __LINE__,__FILE__,A);
#else
#define dprint(A)   
#define gprint(A)   
#define tprint(A)   
#define cprint(A) 
#endif

#ifndef NULL 
   #define NULL ((void *) 0) 
#endif
#ifndef TRUE 
   #define TRUE 1 
#endif
#ifndef FALSE 
   #define FALSE 0 
#endif

#ifndef NSTFREE
   #define NST_FREE(A) if (A) {free(A); A=NULL;}
#endif

#ifndef ABS
#define ABS(X) (((X)<0)?(-(X)):(X))
#endif
#ifndef MIN
#define MIN(a,b) ((a) > (b) ? (b) : (a))
#endif
#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif
#define label_unit(a,b) label(a,b)

#define CALLOC(N,T)  ((N)>0 ? ((T*) calloc((N), sizeof(T))):NULL)
#define REALLOC(P,S) ((S)?((P)?realloc(P,S):malloc(S)):(void*)(free(P),NULL)) /* safe realloc */

#define NST_RETURN    -255  /* this return value of a unit causes termination
                               of the current queue for exec, adapt, ctrl */
#define NST_BREAK     -254  /* this return value of a unit causes termination
                               of the current loop for exec, adapt, ctrl */
#define NST_EXIT      -253  /* this return value of a unit causes return of
                               all nested NST exec/adapt/ctrl calls, i.e.,
                               exit from all current NST calls, but not from the 
                               process. */
#define NST_EXCEPTION -252  /* this return value of a unit indicates that
                               the unit as thrown a NST exception. This modifies
                               subsequent processing to look for a handler unit.  */
#define OK             0    /* @@@@ MG 290492 */

#define NST_REPORT(A,B) if (NST_REPORTS) fprintf(stderr,\
                      "Report from line %d of file %s (date %s):\n   %s\n",__LINE__,__FILE__,B,A);

/* the following are two sets of constants. They are used to
   denote properties of fields, and one set would suffice.
   However, historically these constants are used by different
   structures and routines and must therefore both be there.
   Use the first set in new work.
   [should be straightened in the future]
*/

/* constants for in[]/out[] field descriptor arrays: */

#define F_INPUT  0           /* for field type */
#define F_OUTPUT 1
#define F_CTLFIELD -2
#define F_EPSFIELD -3

/************************************************************************
   define symbolic constants for passing type information to the
   void_unit/primitive_unit routines. Uses large neg int values
   that can never be requested as field dimensions:
*************************************************************************/

#define IMPOSSIBLE_DIM   -99999999   /* used for type specs in arg list */
#define NST_INP       IMPOSSIBLE_DIM 
#define NST_OUT       (IMPOSSIBLE_DIM+1) 
#define NST_HOST      (IMPOSSIBLE_DIM+2) 
#define NST_SPECIAL   (IMPOSSIBLE_DIM+3) 
#define NST_VOID      (IMPOSSIBLE_DIM+4) 
#define NST_SCALAR    (IMPOSSIBLE_DIM+5) 
#define NST_STRING    (IMPOSSIBLE_DIM+6) 
#define NST_BYTE      (IMPOSSIBLE_DIM+7) 
#define NST_INT       (IMPOSSIBLE_DIM+8) 
#define NST_FLOAT     (IMPOSSIBLE_DIM+9)
#define NST_DOUBLE    (IMPOSSIBLE_DIM+10)
#define NST_ANY       (IMPOSSIBLE_DIM+11)
#define NST_END       (IMPOSSIBLE_DIM+11)

extern int NST_VAR;  /* a global variable that holds the currently
                         preferred type. Default is NST_FLOAT. */

/* the following two macros simplify the making of byte and
   integer versions of many units: they temporarily set
   NST_VAR to NST_BYTE or NST_INT, then issue a creation
   call with name postfixed by "8" or "16" and replace
   the exec-routine correspondingly.
*/

#define BYTE_VERSION(U,NAME,ARGLIST) unitptr U;\
   NST_VAR=NST_BYTE; U=NAME ARGLIST; NST_VAR=NST_FLOAT;\
   if (U) U->exec = exec_##NAME##8

#define INT16_VERSION(U,NAME,ARGLIST) unitptr U;\
   NST_VAR=NST_INT; U=NAME ARGLIST; NST_VAR=NST_FLOAT;\
   if (U) U->exec = exec_##NAME##16
                       
                       
/*** second set of definitions for typed pins: [internal use] **********/

#define NST_BYTE_T    'c'
#define NST_INT_T     'i'
#define NST_FLOAT_T   'f'
#define NST_SCALAR_T  's'
#define NST_STRING_T  't'
#define NST_ANY_T     'x'
#define NST_SPECIAL_T '<'   /* starting delimiter of def of SPECIAL_T */
#define NST_ENDDEF_T  '>'   /* to end the definition of SPECIAL_T */
#define NST_VOID_T    'v'


/* bit masks to set/evaluate type-field of  field-struct: */
#define F_IS_OUTPUT  0x1           /* field is output */
#define F_IS_EXPANDABLE 0x10       /* field is expandable */
#define F_HAS_CTLPIN 0x2           /* this field has the ctlpin */
#define nst_has_ctlpin(F) ((F)->property & F_HAS_CTLPIN)

#define is_expandable_f(F) ((F)->property & F_IS_EXPANDABLE)

/********************************************************************

   Utility macros for debugging: 
   to trace function calls for function nameoffunction, insert
   a preprocessor directive 

   #define nameoffunction trace_function(nameoffunction)
 
   in your code. Similarly for the other macros. 
 
********************************************************************/

#define error_spotting(F) (ERROR_LINE=__LINE__,ERROR_FILE=__FILE__,F ## )
#define trace_function(F) (printf(#F " called at line %d (file %s)\n",\
                           __LINE__,__FILE__), F ## )
#define trace_variable(V) (printf(#V "=%10.5f at line %d (file %s)\n",\
                           (float) V,__LINE__,__FILE__), V ## )
#define trace_string(S)   (printf(#S "=%s at line %d (file %s)\n",\
                           S,__LINE__,__FILE__), S ## )

/* macros for checking argument values: */

#define check_positive(N,POS,ROUTINE)\
        ((N>0)?TRUE:error_msg(27,ROUTINE,POS,NULL),FALSE)

#define check_nonnegative(N,POS,ROUTINE)\
        ((N>=0)?TRUE:error_msg(26,ROUTINE,POS,NULL),FALSE)

#define check_numfields(N,POS,ROUTINE)\
        ((N<MAXFIELDS)?TRUE:error_msg(28,ROUTINE,POS,NULL),FALSE)

#define check_ulimit(N,ULIMIT,POS,ROUTINE)\
        ((N<=ULIMIT)?TRUE:error_msg(25,ROUTINE,POS,NULL),FALSE)

#define check_llimit(N,LLIMIT,POS,ROUTINE)\
        ((N>=LLIMIT)?TRUE:error_msg(25,ROUTINE,POS,NULL),FALSE)

#define check_limits(N,LLIMIT,ULIMIT,POS,ROUTINE)\
        ((N<=ULIMIT&&N>=LLIMIT)?TRUE:error_msg(25,ROUTINE,POS,NULL),FALSE)

#define check_unit(U,LLIMIT,ULIMIT,POS,ROUTINE)\
        ((is_unit(U))?TRUE:error_msg(32,ROUTINE,POS,NULL),FALSE)


extern  char *FFMT;
extern  char *NFMT;        /* was IFMT, changed to NFMT [name conflict] */
extern  char *LFMT;        /* default format for writing long ints */

extern  int NST_TRACE;      /* current trace level; O = no traceing */
extern  int NST_REPORTS;    /* if TRUE, enables reports from macro NST_REPORT(..) */
extern  char NST_LINE[255];    /* renamed to NST_LINE */

extern void (*NST_ERROR_PRINTER)(char*);    /* see nst_dbug.c */  
extern void (*NST_ERROR_HANDLER)(int,int);  /* see nst_dbug.c */

extern  int ERROR_LINE;
extern  char *ERROR_FILE;
extern  int ERRORS_FOUND;       /* error counter */
extern  int WARNINGS_FOUND;     /* warnings counter */
extern  void nst_error(const char*,...);
extern  void nst_message(int,const char*,...);
extern  int skip_comments(FILE*,char);
extern  char *str_skip_comments(char*,char);

extern  float RND_LIMIT1;  /* Limit for random initialization */
extern  float RND_LIMIT2;   /* Limit for random initialization */
extern  float RND_PROB;    /* probability for binary rnd-choice */

extern  double nst_drand48(int);   /* uses RND_LIMIT1,2 and RND_PROB */
extern  float gfNstNaN;

/* obsolete variables, but kept for compatibility: */
extern  int TRACE_ON;       /* global trace level variable (valid for all units);
                               if TRUE, report of created connections, and
			       processed units [obsolete, use NST_TRACE below] */
extern  int STEPS;         /* nr of global execution steps performed so far  */
extern  int COMPRESS_ON;            /* if TRUE, writing in compressed way (gzip)
				 reading depends of type of file (compressed 
				 or not) AM */

/* end of obsolete definitions */

extern  void printline(void);
extern  long int nst_ftell(FILE*);
extern  int  fscanf_f(FILE*,float*);
extern  int  fscanf_i(FILE*,int*);
extern  int  fscanf_l(FILE*,long*);
extern  int  nst_tokenize_str(char*,char*,char*,char***);
extern  int  nst_suffix_and_version(char*,char*,int*,int*,int*);
extern  int  nst_paths_are_equal(const char *, const char *);
extern char *nst_expand_file_name(char*,char*);
extern char *nst_dlopen_path(void);
extern void *nst_dlopen(const char*,int);
extern int  nst_dlcall(char*);
extern FILE *nst_fopen(char*, char*);
extern int   nst_fclose(FILE*);
extern FILE *nst_fopen1(char *fn, char *fm, int *iCpr);
extern int   nst_fclose1(FILE *fp, int iCpr);
extern char *nst_dircat(char *pcBuf, ... );
extern char *nst_set_homepath(char*);
extern void  nst_set_additional_path(char*);
extern char *nst_additional_path(void);
extern void  nst_define_filename_substitution(char*,char*);
extern char *nst_exists_filename_substitution(char*);
extern char *nst_find_file_version(char*,char*,char*,char**);
extern int  nst_sscanf_version(char *pcName, int *piMajor, int *piMinor);

extern int nst_printf(const char *pcFmt, ...);
extern int nst_noprintf(const char *pcFmt, ...);
extern int nst_noprintf0(const char *pcFmt, ...);
extern int nst_fprintf(FILE*, const char *pcFmt, ...);
extern int nst_nofprintf(FILE*, const char *pcFmt, ...);
extern int nst_nofprintf0(FILE*, const char *pcFmt, ...);

typedef int (*nst_vprintf_ptr)(const char*, va_list);
typedef int (*nst_vfprintf_ptr)(FILE*, const char*, va_list);

extern nst_vprintf_ptr  nst_set_vprintf(nst_vprintf_ptr);
extern nst_vfprintf_ptr nst_set_vfprintf(nst_vfprintf_ptr);
extern nst_vfprintf_ptr nst_set_vfprintf_for_stderr(nst_vfprintf_ptr);

extern void  nst_fprintf_str(FILE *fp, char *pcStr);
extern char *nst_fscanf_str(FILE *fp, char *pcDest);
extern void  nst_fprintf_flt(FILE *fp, int iNum, ...);
extern void  nst_fprintf_fltvec(FILE *fp, int iNum, float *pfVector);
extern void  nst_fprintf_int(FILE *fp, int iNum, ...);
extern void  nst_fprintf_intvec(FILE *fp, int iNum, int *piVector);
extern void  nst_fprintf_shtvec(FILE *fp, int iNum, short int *piVector);
extern void  nst_fprintf_bytevec(FILE *fp, int iNum, unsigned char *pcVector);
extern int   nst_fscanf_bytevec(FILE *fp, int iNum, unsigned char *pcVector);
extern int   nst_fscanf_fltvec(FILE *fp, int iNum, float *pfVector);
extern int   nst_fscanf_intvec(FILE *fp, int iNum, int *piVector);
extern int   nst_fscanf_shtvec(FILE *fp, int iNum, short int *piVector);
extern int   nst_fscanf_flt(FILE *fp, int iNum, ...);

extern int nst_set_verbose(int i);
extern int nst_verbose(void);

extern void nst_register_argv(int iArgc, char **ppcArgv);
extern int nst_get_argc(void);
extern char **nst_get_argv(void);

extern char *nst_str_from_args(char*,...);  
extern char *nst_filtered_str(char*,char*);
extern char *ParseNstIOStr(char *pcIOStr, int  *piNumInp, char ***pppcInp, 
                           int  *piNumOut, char ***pppcOut);

extern char isBlankField(const char*);
extern int nst_strmatch(const char*,const char*);
extern int nst_strmatch1(const char*, const char*);
extern int nst_strcmp(const char *pc1, const char *pc2, char ch);
extern int nst_strrncmp(const char *pc1, const char *pc2, int n);
extern int ascanf(int argn, char *argv[], char *tFmt, void *pResult);
extern char* parsefield(char* fmt, char *cmd, int *numarg, float *arg1,
                        float *arg2, float *arg3, float *arg4);

/* alias of parsefield: */
extern char* nst_options(char* fmt, char *cmd, int *numarg, float *arg1,
                         float *arg2, float *arg3, float *arg4);
extern char *nst_extract_name(char *pcFmt);
extern  int nst_eval_expr(char*,float*);
extern  double nst_expr_to_double(char*,char**,char*);
extern  int error_report(void);
extern  void nst_usleep(int);
extern  void nst_sleep(float);
extern  int isIntNum(char*);
extern  int isFloatNum(char*);
extern  int getRange(float*,float*,char*,char*);
extern  int getRectCoord(int*,int*,int*,int*,char*);

#define NST_HUGE  99999999 /* pin value beyond which warning is issued */
#define MAXFIELDS 251    /* maximal number of allowed fields plus one */
#define NST_MAXUNITS 99999   /* maximal number of allowed units */

/* Definitions of constants for ctrl_unit routine: */

#define NST_I_RND    (-1) 
#define NST_I_UNIFORM (-1)   /* alias for RND */
#define NST_I_NOR    (-2)
#define NST_I_ZERO   (-3)
#define NST_I_USER   (-4)
#define NST_I_OPEN   (-5)
#define NST_I_CLOSE  (-6)
#define NST_INIT     (-7)    /* constants to switch different modes of the */
#define NST_W_REDRAW (-8)    /* init-routine */
#define NST_W_CLEAR  (-9)
#define NST_W_RESCL  (-10)
#define NST_W_XRESCL (-11)
#define NST_W_YRESCL (-12)
#define NST_W_RESTORE  (-13)
#define NST_W_CLOSE    (-14) /* closes window; window remains attachable */ 
#define NST_W_OPEN     (-15)
#define NST_W_PENUP    (-16)
#define NST_W_PENDOWN  (-17)
#define NST_PLOT_NOBOX (-18) 
#define NST_PLOT_BOX   (-19)
#define NST_W_REMOVE   (-20)  /* stronger than close: removes window */

#define NST_RESET    (-21)
#define NST_REQUEUED (-22)   /* to tell units that they have been requeued */
/* further modes for random initializations: */
#define NST_I_GAUSS    (-23)
#define NST_I_GAUSS_NOR (-24)
#define NST_I_BINARY   (-25)
#define NST_I_SIGN     (-26)

#define NST_W_SETPOS   (-30) /* after load_unit: enforce loaded win position */
#define NST_W_GETPOS   (-31) /* update stored win pos with current win pos */
#define NST_W_SETSIZ   (-32) /* after load_unit: enforce loaded win size */
#define NST_W_GETSIZ   (-33) /* update stored win size with current win size */

#define NST_W_NEWSIZ   (-56) /* (user-triggered) resize notify */
#define NST_W_ORIGSIZ  (-57) /* restore original size */

#define NST_PLOT_XLIN  (-34)
#define NST_PLOT_XLOG  (-35)
#define NST_PLOT_YLIN  (-36)
#define NST_PLOT_YLOG  (-37)
#define NST_W_FIND_HOST (-38) /* find host window & reconfigure */ 
#define NST_F_FLUSH (-39)  /* control mode to flush opened files */
#define NST_EXEC (-40)        /* used by unit ctrl_op in nst_stdr.c */
#define NST_ADAPT (-41)       /* used by unit ctrl_op in nst_stdr.c */
#define NST_INFO  (-45)       /* reserved for future use */
/* modes reserved for window client control functions: */
#define NST_INTERNAL_RESET (-46)
#define NST_INTERNAL_INIT (-47)
#define NST_W_INTERNAL_REDRAW (-48)
#define NST_W_INTERNAL_CLEAR (-49)
#define NST_W_GET_ID (-50)    /* to retrieve window handle */
#define NST_UNBIND   (-51)    /* to break references between var units
                                [should be applied after such unit is moved] */
#define NST_RENAMED  (-52)    /* notify unit when it has become named, or when its
                                 name has been changed or been replaced by a label */
#define NST_W_SAVE_POSINFO (-53)   /* to cause write of window position info */
#define NST_W_LOAD_POSINFO (-54)   /* to cause read of window position info */
#define NST_W_UNPARENT (-55)     /* revert a subwindow to a standalone window */ 

#define NST_THREAD_CANCEL (-60)  /* sent thread cancel request to thread box */ 

/*  -80 .. -89 taken for NST_RPC_TRSP - codes in nst_rpc.h */

#define NST_W_NOEDIT (-100)
#define NST_W_EDIT   (-101)

#define NST_W_SETWIDGETS (-97)

#define NST_T_GET_PINS (-98)
#define NST_T_SET_PINS (-99)

/* some values taken by other packages:
   NST_FREE_WEIGHTS -101  
   NST_PRUNE_WEIGHTS -102
   NST_RANDOMIZE_WEIGHTS -103
   NST_TCL_MAP -270
   NST_TCL_MAP -271
   NST_W_FIND_VIEW3D_WDW -76543
   NST_I_ICON -1123
   NST_UPDATE_ARRAY -1234567
*/

/* Definitions of Pointers for Standard Functions */

typedef float (*nst_func)(float);  /* for the frequent type float(*)(float) */ 

extern  float FUN_GAUSS(float);
extern  float FUN_FERMI(float);
extern  float FUN_DFERMI(float);
extern  float FUN_TANH(float);
extern  float FUN_DTANH(float);
extern  float FUN_EXP(float); 
extern  float FUN_EXPM(float);
extern  float FUN_IDENT(float);
extern  float FUN_ONE(float);
extern  float FUN_SGN(float);
extern  float FUN_CLIP0(float);
extern  float FUN_STEP(float);
extern  float FUN_SIN(float);
extern  float FUN_COS(float);
extern  float FUN_SQR(float);
extern  float FUN_SQRT(float);
extern  float FUN_ABS(float);
extern  float FUN_HAT(float);
extern  float FUN_GAUSS1(float);

/*************************************************************************
   routines adapted from Numerical Recipes  
   [for convenience, when Numerical recipes routines
    are used with NST; error msgs are partially adapted to NST]
**************************************************************************/

extern float  **nst_convert_matrix(float*,int,int,int,int);
extern float  *nst_vector(int,int);
extern double *nst_dvector(int,int);
extern int    *nst_ivector(int,int);
extern float  **nst_matrix(int,int,int,int);
extern double **nst_dmatrix(int,int,int,int);
extern int    **nst_imatrix(int,int,int,int);
extern float  **nst_submatrix(float**,int,int,int,int,int,int);

extern void   nst_free_vector(float*,int,int);
extern void   nst_free_dvector(double*,int,int);
extern void   nst_free_ivector(int*,int,int);
extern void   nst_free_matrix(float**,int,int,int,int);
extern void   nst_free_dmatrix(double**,int,int,int,int);
extern void   nst_free_imatrix(int**,int,int,int,int);
extern void   nst_free_submatrix(float**,int,int,int,int);
extern void   nst_free_convert_matrix(float**,int,int,int,int);
extern int    nst_keystroke(void);
extern int    nst_batchmode(void);
extern void   nst_set_batchmode(int);
extern int    nst_noblock_stdin(void);
extern int    nst_normal_stdin(void);

#endif
