/*****************************************************************************
 *
 * nst1.h  -  NST kernel layer 1: Pins
 *
 * Copyright Helge Ritter, 1992
 *
 * $Source$
 *
 * $Log$
 * Revision 1.5  2000/01/10 17:06:05  helge
 * added code to allow connecting scalar src vector to dyn float dest pin
 * (is enabled when S2DYN is #defined)
 *
 * Revision 1.4  1997/04/29 15:11:08  helge
 * changes necessary with introduction of user-specified data types.
 *
 * Revision 1.3  1997/04/02 14:06:58  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.2  1997/03/13  10:48:27  jan
 * Dependency cleanup.
 *
 *
 *****************************************************************************/

#ifndef __NST1_H
#define __NST1_H
#include "nst_types.h"

/* define some bit masks for the t-element of the pin struct: */
#define NST_W_IS_NEXTPIN 0x1     /* bit 0: indicates successor pin */
#define NST_MULTITYPEPIN 0x2     /* bit 1: indicates non-scalar pin */
#define NST_TYPEMASK     0xEU    /* bits 1-3 indicate type (14=1110): */
#define NST_SCALAR_TYPE    0U    /* the default */
#define NST_FLOAT_TYPE     2U    /* vector */
#define NST_INT_TYPE       4U    /* vector */ 
#define NST_BYTE_TYPE      6U    /* vector; byte = unsigned char */
#define NST_TEXT_TYPE      8U    /* variable length string */
#define NST_UNTYPED       14U    /* untyped pin */


#define has_next_p(P)  (P->t & NST_W_IS_NEXTPIN)
#define is_multitype_p(P) ((P)->t & NST_MULTITYPEPIN)  

extern char t_of_p(pin*);
extern int  dim_of_p(pin*);
extern int  bytesize_of_p(pin*);
extern  pin *mk_typed_p(void*,char,int);  /* to make a typed pin */
extern  pin *alloc_p(pin*,char,int);                           
extern  void free_pin_extension(pin_extension *p);
extern void update_rdrs(pin *p, void *v, int iDim, const char *typestr);

extern  VARTYPE DEFAULT_INPUT0;
extern  VARTYPE DEFAULT_INPUT1;
extern  pin VPIN;

extern int nst_dim_of_p(pin*);
extern int nst_type_of_p(pin*);


extern  void set_adr(pin*,void*);
extern  void *get_adr(pin*);
extern   pin *headr_of(pin*);
extern   pin *succr_of(pin*);
extern   pin *rgrp_of(pin*);
extern   void join_pq(pin*,pin*);
extern   void join_pv(pin*,void*);
extern   void unjoin_pq(pin*, pin*);
extern   void rm_p(pin*);

extern  void del_from_rgrp(pin*);
extern   pin *w_of(pin*);
extern   pin *mk_p(void*);
extern  void print_pin(pin* p);

extern void promote_scalar_p(pin*);
extern void restore_scalar_p(pin*);
extern int compatible_pp(pin *p1, pin *p2);
extern int nst_is_scalar_float_pair(pin *p1, pin *p2);
extern int nst_is_connected_p(pin *p);
extern int define_pin(char cType, int iDim, field *pField, int iPos);
extern void join_pq_0(pin *p, pin *q);
extern int nst_is_connected_inp_p(pin *p);
extern void nst_set_host_of_p(pin *p, field *host);
extern int nst_is_connected_out_p(pin *p);
extern void join_pq_1(pin *p, pin *q);

#endif
