/*****************************************************************************
 *
 * nst3.h  -  NST kernel layer 3: Units
 *
 * Copyright Helge Ritter, 1992
 *
 * $Source$
 *
 * $Log$
 * Revision 1.16  2006/02/19 12:16:24  rhaschke
 * interface of virtual_methods is checked against referenced unit
 * interface of virtual use_methods is not adapted any more
 *
 * Revision 1.15  2005/09/02 13:57:35  rhaschke
 * cleanup in declarations
 * nst_[scrn,X11,prog]: adjustments and additions for NST7's neosrc
 *
 * Revision 1.14  2005/06/06 15:15:43  rhaschke
 * minor formatting changes
 *
 * Revision 1.13  2000/10/16 19:27:13  helge
 * minor changes for compilation with solaris cc -v -Xa
 *
 * Revision 1.12  2000/10/16 17:13:54  helge
 * minor changes for compilation with solaris cc -v -Xa
 *
 * Revision 1.11  2000/06/24 19:20:17  helge
 * defined new mode flag P_ignore_all (to make a unit ignore all operations)
 *
 * Revision 1.10  2000/04/07 14:58:13  helge
 * added definition for flag P_catches_exceptions
 *
 * Revision 1.9  2000/02/07 16:59:53  helge
 * added macro nst_virtual_operands()
 *
 * Revision 1.8  1999/09/27 15:00:40  helge
 * !!! NstInt_t changed from "short int" to "int" to permit
 *     efficient loadlib definitions for int and int* arguments.
 *
 * Revision 1.7  1997/12/21 00:04:54  helge
 * several minor changes
 *
 * Revision 1.6  1997/09/25 11:03:28  helge
 * new function nst_is_named_unit()
 *
 * Revision 1.5  1997/08/23 20:30:57  helge
 * added routines nst_scalar_inp_to_vec, nst_scalar_out_to_vec
 *                nst_vec_to_scalar_inp, nst_vec_to_scalar_out
 * to write between arrays and unpacked fields.
 *
 * Revision 1.4  1997/04/02  14:07:11  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.3  1997/03/20  09:31:53  jan
 * Missing 'extern' declarations added.
 *
 * Revision 1.2  1997/03/13 10:48:33  jan
 * Dependency cleanup.
 *
 *
 *****************************************************************************/

#ifndef __NST3_H
#define __NST3_H
#include "nst_types.h"

extern long glNstQueueChanges;

/* special bits in the property parameter of a unit: */

#define P_trace_level        0xFUL       /* first 4 bits for trace level 0-15 */
#define P_ignore_exec        (0x1UL<<5)  /* mask for exec method */
#define P_ignore_adapt       (0x1UL<<6)  /* mask for adapt method */
#define P_ignore_ctrl        (0x1UL<<7)  /* mask for ctrl  method */
#define P_admits_opnd_offset (0x1UL<<8)  /* indicates operator 
                                            that admits operand offsets */
#define P_debug              (0x1UL<<9)
#define P_do_conversions     (0x1UL<<10) /* indicates need of type conversions */
#define P_uses_vqueue        (0x1UL<<11) /* indicates use of vput/vget
                                            [info used by erasevars routine] */
#define P_is_window_host     (0x1UL<<12) 
#define P_nocheck            (0x1UL<<13)
#define P_is_primitive       (0x1UL<<14) 
#define P_exports_methods    (0x1UL<<15) /* offers callable named subunits */
#define P_is_thread_safe     (0x1UL<<16) /* unit may be used within threads */
#define P_catches_exceptions (0x1UL<<17) /* marks unit as a NST exception handler */
#define P_ignore_all         (0x1UL<<18) /* to ignore all operations: such units
                                            are only for storage of something */

/* some combined masks: */

#define P_ignore_exec_and_adapt  (P_ignore_exec|P_ignore_adapt)
#define P_special_mode  (P_trace_level|P_ignore_exec|P_ignore_adapt|P_ignore_all|P_catches_exceptions|P_debug)

/* some operations on property parameter: */

#define set_property(U,V) ((U)->property |= V)
#define clr_property(U,V) ((U)->property &= ~(V))
#define has_property(U,V) (((U)->property & V) == V)

#define nst_disable_exec_of(u)  (u)->property|=P_ignore_exec
#define nst_disable_adapt_of(u)  (u)->property|=P_ignore_adapt
#define nst_disable_exec_and_adapt_of(u)\
   (u)->property|=P_ignore_exec_and_adapt
#define nst_enable_exec_of(u)  (u)->property&=~P_ignore_exec
#define nst_enable_adapt_of(u)  (u)->property&=~P_ignore_adapt
#define nst_enable_exec_and_adapt_of(u)\
   (u)->property&=~P_ignore_exec_and_adapt
#define nst_exports_methods(u) ((u)->property & P_exports_methods) 
#define nst_is_thread_safe(u) ((u)->property & P_is_thread_safe) 

/* macros to access fields in the unitptr struct: */

#define nst_first_subunit_of(U) (U)->first_subunit
#define nst_last_subunit_of(U) (U)->last_subunit
#define nst_host_of(U) (U)->host
#define nst_next(U) (U)->next
#define nst_prev(U) (U)->prev
#define nst_var_of(U) (U)->var
#define nst_exec_of(U) (U)->exec
#define nst_adapt_of(U) (U)->adapt
#define nst_ctrl_of(U) (U)->ctrl
#define nst_load_of(U) (U)->load
#define nst_save_of(U) (U)->save
#define nst_remove_of(U) (U)->remove
#define nst_inputs_of(U) (U)->ninputs
#define nst_outputs_of(U) (U)->noutputs
#define nst_input_pins(U,F) (U)->in[(F)]->npins
#define nst_output_pins(U,F) (U)->out[(F)]->npins
#define nst_input_f(U,F) (U)->in[(F)]
#define nst_output_f(U,F) (U)->out[(F)]
#define nst_dim_of_f(F) (F)->npins 
#define nst_operands_of(U) (U)->operands
#define nst_first_operand_of(U) (U)->first_operand

/* only to be applied to virtual_for_op and prog_units with virtual_opnd(): */
#define nst_virtual_operands_of(U) (U)->first_operand
/* these units use the first_operand data member to store the nr of outer opnds */

extern void nst_indicate_export_of_methods(unitptr);
extern int add_pins_to_normal_field(int,int,int,int,unitptr);
extern unitptr successor_of(unitptr);
extern unitptr nth_successor_of(unitptr,int);
extern int IsLegalX(unitptr,int,int);
extern int IsLegalY(unitptr,int,int);
extern int IsLegalEps(unitptr,int);
extern int IsLegalCtl(unitptr,int);
extern int nst_can_replace(unitptr,unitptr);
extern int nst_check_interface(unitptr,unitptr);
extern void nst_swap_subset_of_corresponding_pins(unitptr,unitptr);

#define inputs_of(U) U->ninputs
#define outputs_of(U) U->noutputs
#define dim_of_X(P,I) ((P)->in[(I)]->dim)
#define dim_of_EPS(P) ((P)->in[(P)->ninputs-2]->dim)
#define dim_of_CTL(P) ((P)->in[(P)->ninputs-1]->dim)
#define dim_of_Y(P,I) ((P)->out[(I)]->dim) 

/* two dummy pins to absorb values in case of errors: */
extern pin *NST_DUMMY_PIN, *NST_DUMMY_PIN1; 

/* some of the macros below contain apparently redundant casts
   of the form *((type**) &ptr) to make them useable as lvalues
*/
#define pNstInp(U,F,P)  ((U)->in[F]->p[P])
#define pNstOut(U,F,P)  ((U)->out[F]->p[P])
#define pvNstInp(U,F,P) (*((void**)&((U)->in[F]->p[P]->s)))
#define pvNstOut(U,F,P) (*((void**)&((U)->out[F]->p[P]->s)))
#define fNstInp(U,F,P)  ((U)->in[F]->p[P]->s[0])
#define fNstOut(U,F,P)  ((U)->out[F]->p[P]->s[0])
#define pcNstInp(U,F,P) (*((unsigned char**)&((U)->in[F]->p[P]->s)))
#define pcNstOut(U,F,P) (*((unsigned char**)&((U)->out[F]->p[P]->s)))
#define piNstInp(U,F,P) (*((NstInt_t**)&((U)->in[F]->p[P]->s)))
#define piNstOut(U,F,P) (*((NstInt_t**)&((U)->out[F]->p[P]->s)))
#define pfNstInp(U,F,P) (*((float**)&((U)->in[F]->p[P]->s)))
#define pfNstOut(U,F,P) (*((float**)&((U)->out[F]->p[P]->s)))
#define ptNstInp(U,F,P) (*((char**)&((U)->in[F]->p[P]->s)))
#define ptNstOut(U,F,P) (*((char**)&((U)->out[F]->p[P]->s)))

extern void nst_def_ctlpin(unitptr,int,int);
extern void nst_init_ctlpin(unitptr,int,int);
extern void nst_disable_operands_of(unitptr);

/* some utility functions for transfers between scalar fields
   and linear float/int/byte arrays: */

extern void nst_scalar_inp_to_vec(unitptr, int, float*);
extern void nst_scalar_out_to_vec(unitptr, int, float*);
extern void nst_vec_to_scalar_inp(float*, unitptr, int);
extern void nst_vec_to_scalar_out(float*, unitptr, int);

extern void nst_scalar_inp_to_float_vec(unitptr, int, int, int, float*);
extern void nst_scalar_out_to_float_vec(unitptr, int, int, int, float*);
extern void nst_scalar_inp_to_int_vec(unitptr,   int, int, int, int*);
extern void nst_scalar_out_to_int_vec(unitptr,   int, int, int, int*);
extern void nst_scalar_inp_to_byte_vec(unitptr,  int, int, int, unsigned char*);
extern void nst_scalar_out_to_byte_vec(unitptr,  int, int, int, unsigned char*);

extern void nst_float_vec_to_scalar_inp(int, float*, unitptr, int, int);
extern void nst_float_vec_to_scalar_out(int, float*, unitptr, int, int);
extern void nst_int_vec_to_scalar_inp(  int, int*,   unitptr, int, int);
extern void nst_int_vec_to_scalar_out(  int, int*,   unitptr, int, int);
extern void nst_byte_vec_to_scalar_inp( int, unsigned char*, unitptr, int, int);
extern void nst_byte_vec_to_scalar_out( int, unsigned char*, unitptr, int, int);

#ifdef NST_DEBUG 

/* (installed by make_ROOT()) */
extern int IsLegalVal;

#define X(U,F,I) (ERROR_LINE=__LINE__,ERROR_FILE=__FILE__,((IsLegalX(U,F,I))?((U)->in[F]->p[I]):NST_DUMMY_PIN)->s[0])
#define Y(U,F,I) (ERROR_LINE=__LINE__,ERROR_FILE=__FILE__,((IsLegalY(U,F,I))?((U)->out[F]->p[I]):NST_DUMMY_PIN)->s[0])
#define XP(U,F,I) (ERROR_LINE=__LINE__,ERROR_FILE=__FILE__,((IsLegalVal=IsLegalX(U,F,I))?((U)->in[F]->p[0]):NST_DUMMY_PIN1)->s[(I)*IsLegalVal])
#define YP(U,F,I) (ERROR_LINE=__LINE__,ERROR_FILE=__FILE__,((IsLegalVal=IsLegalY(U,F,I))?((U)->out[F]->p[0]):NST_DUMMY_PIN1)->s[(I)*IsLegalVal])
#define EPS_in(U,F) (ERROR_LINE=__LINE__,ERROR_FILE=__FILE__,((IsLegalEps(U,F))?((U)->in[(U)->ninputs-2]->p[F]):NST_DUMMY_PIN)->s[0])
#define CTL_in(U,F) (ERROR_LINE=__LINE__,ERROR_FILE=__FILE__,((IsLegalCtl(U,F))?((U)->in[(U)->ninputs-1]->p[F]):NST_DUMMY_PIN)->s[0])
#else
#define X(U,F,I)  ((U)->in[F]->p[I]->s[0])
#define Y(U,F,I)  ((U)->out[F]->p[I]->s[0])
#define XP(U,F,I) ((U)->in[F]->p[0]->s[I])
#define YP(U,F,I) ((U)->out[F]->p[0]->s[I])
#define EPS_in(U,I) ((U)->in[(U)->ninputs-2]->p[I]->s[0])
#define CTL_in(U,I) ((U)->in[(U)->ninputs-1]->p[I]->s[0])
#endif

#define in_(P,I,J) X(P,I,J)
#define out_(P,I,J) Y(P,I,J)

#define in_P(P,I,J) XP(P,I,J)
#define out_P(P,I,J) YP(P,I,J)

/* special cases: */

#define X_in(P,J) X(P,0,J)
#define Y_in(P,J) X(P,1,J)
#define Z_in(P,J) X(P,2,J)

#define X_out(P,J) Y(P,0,J)
#define Y_out(P,J) Y(P,1,J)
#define Z_out(P,J) Y(P,2,J)

#define XP_in(P,J) XP(P,0,J)
#define YP_in(P,J) XP(P,1,J)
#define ZP_in(P,J) XP(P,2,J)

#define XP_out(P,J) YP(P,0,J)
#define YP_out(P,J) YP(P,1,J)
#define ZP_out(P,J) YP(P,2,J)

/* special cases, not yet included in error checking: */

#define EPSP_in(P,J) (P)->in[(P)->ninputs-2]->p[0]->s[(J)]
#define CTLP_in(P,J) (P)->in[(P)->ninputs-1]->p[0]->s[(J)]

extern  void describe_pin(pin*);
extern  void describe_queue(unitptr);
extern  int describe_unit(unitptr, int);
extern  void diagnose_connectivity(unitptr u);

extern  int nst_exec_container(unittype*);
extern  int nst_exec_void_unit(unittype*);
extern  int nst_exec0(unittype*);
extern  int nst_remove0(unittype*);
extern  int nst_adapt0(unittype*);
extern  int nst_ctrl0(int,unittype*);
extern  int nst_load0(FILE*, unittype*);
extern  int nst_save0(FILE*, unittype*);

extern  int EXEC_DEFAULT(unittype*);
extern  int DELETE_DEFAULT(unittype*);
extern  int ADAPT_DEFAULT(unittype*);
extern  int CTRL_DEFAULT(int,unittype*);
extern  int LOAD_DEFAULT(FILE*, unittype*);
extern  int SAVE_DEFAULT(FILE*, unittype*);

extern  unitptr mk_u(int *, int*, unitptr);
extern  void rm_u(unitptr);
extern  unitptr make_ROOT(void);
extern  unitptr void_unit1(int*, int*, unitptr);
extern  unitptr primitive_unit1(int*, int*, unitptr);
extern  unitptr void_unit(int,...);
extern  unitptr primitive_unit(int,...);

extern int expand_field(field*,char*,...);
extern  unitptr void_unit2(int, ...);
extern  unitptr primitive_unit2(int, ...);
extern  unitptr void_unit3(int,char*[], int,char*[], unitptr);
extern  unitptr primitive_unit3(int,char*[], int, char*[], unitptr);

extern  unitptr reflect(int,char*,unitptr);
extern  unitptr reflect8(int,char*,unitptr);
extern  unitptr reflect16(int,char*,unitptr);
extern  int* pinfield(int);
extern  int nst_swap_units(unitptr,unitptr);
extern  int nst_swap_units1(unitptr,unitptr,int);
extern  int eq_u(unitptr,unitptr);
extern  int queue(unitptr,int);
extern  int queue_unit(unitptr,int);
extern  void unqueue(unitptr);
extern  void change_host(unitptr,unitptr);
extern  void nst_change_host(unitptr,unitptr);
extern  void nst_change_host0(unitptr,unitptr);
extern  void set_defaults(unitptr);
extern  void label(unitptr,const char*);
extern  int legalpin(unitptr,int*,int);
extern  int legalfield(unitptr,int*);
extern  int is_container_unit(unitptr);
extern  int nst_is_named_unit(unitptr);
extern  int nst_is_name(const char*);
extern  int is_not_unit(unitptr);
extern  unitptr u_of_f(field*);
extern  unitptr u_of_p(pin*);
extern  int id_of_f(field*);
extern void zero_Y(unitptr,int);

#endif
