/*****************************************************************************
 *
 * nst9.h  -  NST kernel layer 9: support for user-specified NST types
 *
 * Copyright Helge Ritter, 1997
 *
 * $Source$
 *
 * $Log$
 * Revision 1.8  2005/09/02 13:57:33  rhaschke
 * cleanup in declarations
 * nst_[scrn,X11,prog]: adjustments and additions for NST7's neosrc
 *
 * Revision 1.7  2002/03/22 11:34:22  rhaschke
 * added nst_copy_posinfo ()
 *
 * Revision 1.6  2002/02/06 16:07:47  rhaschke
 * added base class to user-defined pin types
 * allows to connect a derived class to a base class pin (if compatible)
 *
 * Revision 1.5  2000/06/24 19:25:18  helge
 * CHANGE: nst_write_posinfo/nst_read_posinfo now save/restore also open/close
 *      status of a window from/to file. Old circuit files can still be read,
 *      newly written files will require NST 7.3.
 *
 * Revision 1.4  1999/06/16 15:14:49  helge
 * added nst_typedef_description() to allow registration of a "description
 *    method" for each user-defined type;
 * added nst_description_of_typed_pin() to retrieve description of data in
 *    a pin with user-defined type if decription method has been registered.
 * nst_typedef(name,NULL,NULL,NULL) now unregisters a previously def'd type.
 * NST_TYPEDEF() macro now in header
 *
 * Revision 1.3  1998/01/05 18:20:36  helge
 * various changes to support new C/C++ object wrapping scheme with
 * nested class containers; now also packed float fields dim=1 in
 * most cases equivalent to a single scalar float.
 *
 * Revision 1.2  1997/05/06 07:23:49  jan
 * Added crucial newline at end of file.
 *
 * Revision 1.1  1997/04/29 15:04:22  helge
 * adds nst_typedef() function to extend NST pin types by user-defined
 * data types.
 *
 *****************************************************************************/

#ifndef __NST9_H
#define __NST9_H
#include "nst_types.h"

extern void nst_typedef(char *pcName, 
   int   (*pCompatible)(void*,void*), 
   void *(*pCopy)(void*,void*), 
   void  (*pFree)(void*)
);

#define NST_TYPEDEF(T,A,B,C) \
nst_typedef(T, (int(*)(void*,void*)) A,(void*(*)(void*,void*)) B,(void(*)(void*)) C)

extern int nst_functions_of_type(char *pcName,
                                 void **ppCompatible,
                                 void **ppCopy,
                                 void **ppFree);

extern void nst_typedef_description(char *pcName, 
                                    char *(*pDescribe)(void*));
extern char *nst_description_of_typed_pin(pin *pPin);

void nst_typedef_basetype (char *pcName, char *pcBase);

extern int   nst_typedef_for_join(pin *pSrc, pin *pDest);
extern int   nst_typedef_for_unjoin(pin *pSrc, pin *pDest);
extern void  nst_typedef_for_p(pin  *pPin, char *pcName);
extern char *nst_typename_of(pin *p);
extern int   nst_compatible_void_pp(pin *p1, pin *p2); 
extern int   nst_is_typeless_void(pin *p);
extern void  nst_free_pindata(pin *p);
extern void *nst_copy_void_p(pin *pTo, pin *pFrom); 
extern unitptr cast_unit (const char *pcFrom, const char *pcTo, 
                          unitptr uHost);

extern int  *nst_make_posinfo(void);
extern int  *nst_copy_posinfo(int*);
extern int  *nst_read_posinfo(FILE*, int* piXll, int* piYll,
                              int* piDx, int* piDy, int* piOpen);
extern int   nst_write_posinfo(FILE*, int iXll, int iYll,
                               int iDx, int iDy, int iOpen, int*);
#endif
