/* 
 * $Log$
 * Revision 1.3  1997/04/02 14:08:24  helge
 * most man pages now in source code; some corrections;
 * for list, check out new log file in main dir
 *
 * Revision 1.2  1997/03/13  10:54:28  jan
 * Dependency cleanup.
 *
 * Revision 1.9  1997/01/20 13:46:36  helge
 * various minor changes.
 *
 * Revision 1.8  1996/08/07  14:22:11  helge
 * New NST-Version 4.8 / Neo-Version 2.8  accumulating major changes
 *
 * Revision 1.7  1995/07/31  09:22:18  helge
 * some minor changes.
 *
 * Revision 1.5  1995/07/16 20:54:27  helge
 * enhanced rpc_unit to make remote units from neo's *.NST-files
 * extended nst_rpd.c, nst_rpd_add accordingly [changes disabled by
 * undefing NEO in nst_rpd_add.c]
 * provided 8-bit versions of some nst-units [nst_math8 etc.]
 *
 * Revision 1.4  1995/04/24  11:05:02  mkrause
 * some warnings removed
 *
 * Revision 1.3  1994/11/10  11:09:59  walter
 * NOMAIL, Log++
 *
 */

#ifndef __NST_AUX6_H
#define __NST_AUX6_H

#include <stdio.h>

extern int _iLastFmt_;  /* last format sent */
extern float XLLCM, YLLCM, DXCM, DYCM;

#define FMT_ERR  0 /* format error, MUST BE ZERO */
#define FMT_PPM -1
#define FMT_PGM -2
#define FMT_VEC -3
#define FMT_TXT -4
#define FMT_HDR -5   /* indicates header */
#define FMT_EOR -6   /* marks end of record */
#define FMT_EOS -7   /* marks end of stream */

/* macros for list processing: the following macros implement
   some basic list processing operations, assuming a singly
   linked list with linking elements p->next. */

/* LIST_LOOP: to loop over the elements of a list starting at pList
   (pList will be NULL afterwards): */

#define LIST_LOOP(pList) for (;pList!=NULL;pList=pList->next) 

/* LIST_DELETE: to delete the first element of a list starting at pList
   that satisfies EqCondition. The pointer pList will visit
   all elements in the list and EqCondition must be a logical
   expression using pList. After return, pList has its previous
   value restored. The beginning of the list does not change
   (even when the first el is deleted)
*/
 
#define LIST_DELETE(List_t,pList,EqCondition) {List_t *_pL_,*_pL1_;\
if(pList){if(EqCondition){\
  if ((_pL_=pList->next)){memmove(pList,_pL_,sizeof(List_t));free(_pL_);}\
} else {for (_pL1_=pList;_pL_=pList,pList=pList->next;){\
    if(EqCondition){_pL_->next=pList->next;free(pList);break;}\
} pList=_pL1_;}}}

/* LIST PREPEND: to prepend a new element at the start of a list.
   After execution, pList will be the new start of the list
   (may have changed!). 
*/

#define LIST_PREPEND(List_t,pList){List_t *_pL_;\
   _pL_=CALLOC(1,List_t);_pL_->next=pList;pList=_pL_;}

/* LIST_SEEK: sets pList, which must point to a list element,
   to the first list element that follows after pList in the
   list and that satisfies EqCondition. EqCondition must be a 
   logical expression that uses pList. pList=NULL indicates
   that such element could not be found.
*/

#define LIST_SEEK(List_t,pList,EqCondition)\
   LIST_LOOP(pList)if(EqCondition)break;

#define MAXNAMELEN 200

typedef struct sFpList_{
   FILE *fp;
   char acName[MAXNAMELEN];
   struct sFpList_ *next;
} sFpList;

extern sFpList *_FILE_FP_LIST_;

FILE *FOPEN_(char *pcName, char *pcMode); 

void FCLOSE(FILE *fp);

char *FileNameOf(FILE *fp);

typedef struct Dap_t_{
    char cType;  /* contains type info: i,f,c for int, float, char */
    int *piDim;  /* pointer to size info vector */
    union {int *pi; float *pf; char *pc;} u;
    /* contains pointers to reference data */
    char *pcBuf; /* pointer to internal data buffer.
                    All free's, mallocs and reallocs are restricted
                    to this buffer. They never act on externally
                    referenced data. */
    struct Dap_t_ *next; /* link to next list element */
} Dap_t;

int isTxt(Dap_t*);
int isHdr(Dap_t*);
int isVec(Dap_t*);
int isPic(Dap_t*);
int isRgb(Dap_t*);
int isMono(Dap_t*);
char *RmEndDigits(char *pcName);
char *GetEndDigits(char *pcName);
void GetSourceFile(Dap_t *psData, char *acDir, char *acName, char *acType);
void SetSourceFile(Dap_t *psData, char *pcPath, char *pcFile, char *pcSuffix);
void AppendComment(Dap_t *psData, char *pcComment);
void StripComments(Dap_t *psData); 

int iReadPxmHeader(FILE *fp, char *pcHeader, int *piDx, int *piDy,
                   char *pcMagicString, int *piRange);

void iReadPxmData(FILE *fp, char *acType, int iDx, int iDy,
           int iXll, int iYll, int iWidth, int iHeight,
           float *pfR, float *pfG, float *pfB);

void fprintFmt(FILE *fp, int iFmt);
int iFormat(char *acBuf);

Dap_t *SetDapType(Dap_t*,int);

int iReadDap(FILE  *fp,     /* input stream */
          Dap_t *psData  /* destination address */);

void WriteDap(FILE  *fp,     /* output stream */
           Dap_t *psData  /* source data */);

Dap_t *RefPic(Dap_t *psData, int *piDx, int *piDy, 
           float **ppfR, float **ppfG, float **ppfB, char **pcTitle);

Dap_t *DefPic(Dap_t *psData, int iDx, int iDy, 
           float *pfR, float *pfG, float *pfB, char *pcTitle);

Dap_t *NewPic(int iDx, int iDy,
           float *pfR, float *pfG, float *pfB, char *pcTitle);

Dap_t *RefVec(Dap_t *psData, int *piDim, float **ppfVec, char **ppcTitle);
Dap_t *DefVec(Dap_t *psData, int iDim, float *pfVec, char *pcTitle);
Dap_t *NewVec(int iDim, float *pfVec, char *pcTitle);

Dap_t *RefTxt(Dap_t *psData, char **ppcTxt);
Dap_t *DefTxt(Dap_t *psData, char *pcTxt);
Dap_t *NewTxt(int iLen, char *pcTxt);

Dap_t *GetPic(Dap_t *psData, int *piDx, int *piDy, 
           float *pfR, float *pfG, float *pfB, char *pcTitle);

Dap_t *SetPic(Dap_t *psData, int iDx, int iDy, 
           float *pfR, float *pfG, float *pfB, char *pcTitle);

Dap_t *GetVec(Dap_t *psData, int *piDim, float *pfVec, char *pcTitle);
Dap_t *SetVec(Dap_t *psData, int iDim, float *pfVec, char *pcTitle);

void FreeDap(Dap_t *psData);

Dap_t *MakeMono(Dap_t *psData);
Dap_t *NewDap(Dap_t *psData);

FILE *FOPEN(char*,char*);
void PRINTF(char*, ...); 
void FPRINTF(FILE*, char*, ...);

#endif /* header control */
