/***********************************************************************
 * include file
 * Merged version of psom.h and psom2.h
 * $Source$
 * $Log$
 * Revision 1.9  2000/10/16 19:42:24  helge
 * removed warning
 *
 * Revision 1.8  1997/12/04 11:10:05  walter
 * textual interface change:
 * -limit .<value>:   set extrapolation limit value (relative to s-range)
 * -abslimit .<value>:   set extrapolation limit value absulute [-value...+value]
 *
 * Revision 1.7  1997/07/30 16:15:04  walter
 * recognize NST_I_USER in order to complete X_out on Y_in -> display manifold
 * call operands on each interation step.  -> visualize iterations
 * save option for desc-block-file -> new neo-interfaces
 *
 * Revision 1.6  1997/06/20 19:31:24  walter
 * Fixed bugs, merged code, incl. multi-start
 * check on limit-policy
 *
 *
 ***********************************************************************/

#ifndef __NST_PSOM_H
#define __NST_PSOM_H
#ifdef SGI
#include <bstring.h>
#endif

#include "nst_psom_aux2.h"

#define PSOMMAXDIM	12		/* we don't take more, JW*/

#define fmPSOM2_Lambda0(u)	X((u),3,0) /* initial Levenberg-Marquart */
#define fmPSOM_Eps2_in(u)	X((u),3,1)
#define fmPSOM_ErrLim(u)	X((u),4,1) /* tolerated error limit */
#define fmPSOM_MaxIter(u)	X((u),4,2) /* max num of iterations */
#define fmPSOM2_MaxIter(u)	X((u),4,2) /* max num of iterations */
#define fmPSOM2_ExecMode(u)     X((u),4,3) /* exec mode <0> */
#define fmPSOM2_AdjLambda(u)	X((u),4,4) /* auto-adjust Lambda */


#define fmPSOM_Iter(u)		Y((u),3,0) /* num of iterations */
#define fmPSOM2_LambdaOut(u)	Y((u),3,1) /* last Lambda used */
#define fmPSOM_xyDist(u)	Y((u),3,2) /* dist in input subspace */
#define fmPSOM_DeltaSabs(u)	Y((u),3,3) /* last step in map coord */
#define fmPSOM_KnotsXvolDiag(u) Y((u),3,4) /* DiffMinMaxKnotDist, update mode 0 */
#define fmPSOM_ScaleXaxis(u,i)	Y((u),4,(i)) /* component scale factor, auto=1/range**2 */

#ifndef _fmONLY_NEW
 #define fmPSOM_Eps1_in(u)	X((u),3,0) /* modified meaning see above */
 #define fmPSOM_Eps1_out(u)	Y((u),3,1) /* last eps1 used */
 #define fmPSOM_Eps1AutoAdj(u)	X((u),4,4) /* for compatibility */
#endif

extern unitptr	psom2(int iXdim_, int iMdim_, int aiKnotMax_[], 
    float *pafKnotVal_[], int *piUseMaxKnot_, char * tOtions, unitptr host);
extern void	psomSetHypo(unitptr u, int *piUseMaxKnot_);

extern int gPSOM_use_3[];
/* in nst_psom_aux3.c: */
extern unitptr image_psom( 
	char *tMask ,	/* mask string defining input and output pins */
	unitptr uPsom, /* unitptr of existing psom unit */
	unitptr uHost /* host unit */ 
    );
extern int setupMetaPsom(unitptr uM, unitptr u, int iMetaEmbRestDimIgnore);
extern int trainMetaPsom(unitptr uM, unitptr u, float *pfRest);
extern int recallMetaPsom(unitptr uM, unitptr u, float *pfRest);
extern int	cpMetaPsom2Tpsom(FILE *fp, unitptr u, unitptr uM);
	 /* copy weigths from metaPsom uM according to T-Psom  */
extern int	checkMetaPsom2Tpsom(char * acCom, unitptr u, unitptr uM);

#ifdef INC_PSOM_H_ALL		/* do not export all */

struct _sPsomAxis {		/* all for this s-axis only */
	int	iId;		/* identifier number */
	char 	*tId;		/* identifier string */
	int	iKnots;		/* knots in this axis */
	double	*pdVal;		/* knot values */
	double  dMeanValDist;	/* mean dist between knot vals */
	double	dS;		/* s value (can be the source for computing) */
  long unsigned	lOnSet;		/* bit mask 1=on 0=off */
  	int	iFirstIdx;	/* allows for( i=iFirstIdx; i< iEndIdx; i++) */
	int	iEndIdx;	/*   call dPsomAxisModSet() to update */
	double	*pdPDa;		/* prod Diff a[]-a[i] for the lOnSet  */
	double	*pdL;		/* Lagrange Multi L_{n,[i]}(s) 	[iKnots] */
	double	*pdDL;		/* dL_{n,[i]}/ds 		[iKnots] */
	double	*pdD2L;		/* d2L_{n,[i]}/d2s (='')	[iKnots] */
	int	iJ;		/* '[nu] := argmin_j a[nu][j]-s[nu], index */
	double	*pdDs;		/* Diff s-a[] */
	float	**ppfS;		/* source pointer to fS, 
				 * e.g. =&(uPsom->out[1]->p[iId]->s) 
				 * (can be the source for computing) */
};
typedef struct _sPsomAxis SPsomAxis;


struct _sPsomRank {		/* rank structure */
	int	iNumRank;	/* number best ranks */
	float   fExcusionWeightFactor; /* %*% number to add to norm dist */
	float   fCostAccept;	/* threshold to accept result anyway*/
	int	*piA;		/* best ranks */
	int     iA;		/* last best rank */
	float   *pfS;		/* s vector */
	float   fDist;		/* normed dist in input space */
	float   fExc;		/* excursion cost function  */
};
typedef struct _sPsomRank  SPsomRank ;


struct _PSOM_STRUCT {			/* tmp for extensions */
	char 	*	tId;		/* unit-name */
	int		iXdim;		/* copies for extern access */
	int		iMdim;
	int		iKnotTotNum;
	int	*	piKnotMax;	/* ==aiKnotMax */
	float 	*	pfDeltaXY;	/* ==aiDelta	*/
	SPsomAxis*	axes;		/* lagrange Axes */
	float	**	ppfW_a;		/* pnt array to reference vectors
						W[a]=W[0..iKnotTotNum-1] */
	DSVD_VARS_STRUCT* sDsvd;	/* svd stuff on afAlpha[] */
	double	*	pdBestAlpha;	/* best lin sys */
	double	*	pdBestBeta;	
	double	*	pdAlpha;	/* lin sys */
	double	*	pdBeta;	
	double	*	pdDeltaS;	/* result of lin sys */
	int 	*	piUseMaxKnot;	/* NULL or ai[iMdim] */
	unsigned long *	plUseKnot;	/* NULL or al[iMdim] */
	unsigned long *	plUseKnotBest;	/* NULL or al[iMdim] */
	int		iMaxShiftCnt;	/* dyn centering sub knot set, max */
	int	*	piShiftCnt;	 /* hypo psom shift counter */
	int	*	piShiftCntBest;	 /* hypo psom shift counter */
	float 		fBorderKnotRetractFactor; /* [0..~0.5], default 0 */
	int		iExtraModes;	/* some extras */
	char	*	tPinDef;	/* string[iXdim] 'i|o|O' O=adapt*/
	SPsomRank *     psRank;	        /* rank stuff  */
        int             opndMode;       /* operand modes */
  float * pfSlimitMin;		/* s-vector-bounds */
  float * pfSlimitMax;
  int     iClipped;		/* set if S-bounds reached */
  float   fExtrapolationLimit;  
            /* extrapolation limit in s-coord. Can be set with new option 
               e.g. -limit 1.2  yields 20 % range limit beyond each dir,
                    -limit 1 fixes returned pnt to s-cuboid */
    float   fSigma2;  /* if nonzero, gaussian basis functions are used
                         Activate with option -gauss fSigma */
};
typedef struct _PSOM_STRUCT PSOM_STRUCT;


extern int	boundXdim(void);
extern int	unboundXdim(void);
extern int psomKnotNum2s_Retr(float * pfS, float fBorderRetract, int iKnotNum);
extern void psomS2Y(float *pfS);
extern SPsomRank *	psomNewRank(int iNumRank);
extern void psomFreeRank(SPsomRank * ps);
extern void psomMultiSetup(int iNumRank, float fExcWeightFac, float fCostOk);
extern float psomRankWeightExcursion(void);     
extern int exec_psom2_rank(unitptr u);


#define LAGRANGEMAX PSOMMAXDIM




extern SPsomAxis	*asPsomAxisArrayByCurrPsom(void);
extern SPsomAxis	*psPsomAxisFill(SPsomAxis *ps, int iN, float *pfVal);
extern SPsomAxis	*asPsomAxisArrayNew(int iM, int *piN, float *apVal[]);
extern SPsomAxis	*setCurrPsomAxisArray(SPsomAxis *as);
extern SPsomAxis	*psomAxisEmptyStruct(SPsomAxis*ps);
extern void  	psomAxisFreeArray(SPsomAxis *as);
extern double 	dPsomAxisModSet(SPsomAxis *psA, long unsigned lDoSet, int iMode);
extern int 	iPsomAxisComp(SPsomAxis *psA, long unsigned lDoSet,int iMode);
extern int 	iKnotIsOn( int *pIdx  /* knot index vector */ );
extern double 	dPsomAxisArrComp_B(int *pIdx,long unsigned *plDoSet,int iMode);
extern int iPsomAxisArrComp_BD2( 
	float	*pfBa,		/* Ba(s) */
	float	*pfDBaDs,	/* first derivatives */
	float	*pfD2BaDsDs, 	/* second derivatives */
	int	*pIdx,		/* index vector specifying knot */
  long unsigned	*plDoSet,	/* defines set of knots by bit value */
	int	iMode		/* mode */
    );
extern int iPsomAxisArrComp_BD2_fast( 
	float	*pfBa,		/* Ba(s) */
	float	*pfDBaDs,	/* first derivatives */
	float	*pfD2BaDsDs, 	/* second derivatives */
	int	*pIdx 		/* index vector specifying knot */
    );
	
extern int psomAxisArrComp_BaVect( float *pfBa, long unsigned *plDoSet, int iMode);

extern void	setCurrPsom(unitptr u);
extern void	psomAxisSetKnotVal(int iCmd, float **ppf);
extern float	psomUniScale(void);
extern void	aiKnotCnt_by_a(int iA);
extern void	psomGetOnSetByInput( void );
extern void	psomGetOnSetByBestMatch(int iBestA);
extern void	psomY0DiffByBa( void );
extern char *	tLong2Str(
	char	*tS,	   /* return string, if(==0) return static char* */
  long unsigned	l,	   /* long convertet to binary format */
	int 	iMaskWidth /* field width, if(==0) take all */
		);
extern int	iArrOffsetFd(double *pdX, float fX, int iN, int iNmax, int iLMR);
extern PSOM_STRUCT * psomGetV(void)	;

extern int axisCheckSetCentered(SPsomAxis *psA, int iStart, int iLen);
extern int axisCheckSetExceed(SPsomAxis *psA, int iPax_c_ret) ; 
extern int axisShiftAllSetsCentered(void);

extern int setCurrPsom_test(unitptr);
extern int psomFindBestVector(float*, int *, int );
extern int axisPercentSetExceed(SPsomAxis*);
extern int psomPrintBa(int);
extern int fprintCurrPsomDesc(FILE*,float*,int,char*);


#endif   /*^^^^^^^^^^^^^ END OF PSOM PRIVATE:  INC_PSOM_H_ALL */



/* iMode Bit defines:
    If ( .iMode & PSOMAXISSRCINTERN ) current s value is taken from intern psA->dS else from **psA->ppfS.
*/
#define PSOMAXISSRCINTERN	0x1	/* s-value soucre is struct value, already set */
#define PSOMAXISCLEAROFFKNOTS	0x2	/* blank non-OnSet knots */
#define PSOMAXISCLEAROFFSRUCT	0x4	/* blank non-OnSet struct values */
#define PSOMAXISONLY_L		0x800	/* compute only pdL[] */
#define PSOMAXISVERBOSE_0	0x10	/* verbose dPsomAxisModSet() */
#define PSOMAXISnoEXITonERROR	0x100	/* will set global in dPsomAxisModSet() */ 

#define PSOM_OPND_COMPLETE      0x1     /* will complete Xout and unweighted xyDist(u) */
#define PSOM_OPND_INDICATE_BACKSTEP 0x2 /* call opnd twice on bad iterations */
#define PSOM_OPND_NO_PEN_CTRL   0x4     /* do not issue PENUP / PENDOWN ctrl-calls */

extern int PSOM_EXIT_ON_ERROR;


extern FILE * setCurrentTracePsomFP(FILE * );
extern FILE * getCurrentTracePsomFP(void);
extern FILE * setTracePsomFP(FILE * fp, unitptr u);
extern FILE * getTracePsomFP(unitptr u);
 


extern int psomConvStr2Z(unitptr u, char *tMask, char tBook[]);
	/*
	 * convert tMask by code book tBook and adjust Z_in(u) 
	 * returns 0 on success: or mask[pos] with does not match 
	 * tBook==NULL? take default '01oiOI><' (even->0, odd->1) 
	 */

extern char* psomConvZ2Str(unitptr u, char *tMask, char tSet2[]);
	/*
	 * convert Z_in mask to ALREADY allocated tMask by code book tSet
	 * 0 -> tSet[0] 1->tSet2[1]    tSet==NULL take default '01'
	 * returns pointer to tMask0 
	 */

extern int	psomSetInByStr(char *tPin);
	/*
	* convert tPin by adjust Z_in(uCurrent) 
	* if tPin to short the rest is filled with 0
	* returns # of inputs set, neg if bad error
	*/
extern int	psomUpdateStrByIn(void);
	/*
	* convert Z_in(uCurrent) to updated  gpV->tPinDef
	* oO only changed if needed
	*/
extern int iPsomFixKnotsArray(int iMapDim, int *piKnotsSrc, int *piKnotsDest, 
	    int *piKnotsTotNum, int iVerbose);
	/*
	 * Eliminates Knots dimensions i with piKnotsSrc[i]==1 knots
	 * Returns fixed piKnotsDest Array and the fixed mapping dimension
	 * return -1 on errors like piKnotsSrc[]<1 
	 * if (iVerbose==0) is operates silently 
	 */




extern int savePsomBlock(
	FILE *fp, /* file pointer to append to */
	unitptr uPsom,		/* psom unit */
	int iLenRest,		/* Length of afRest */
	float *afRest,		/* float array */
	char* tCommentBlock,	/* comment (CR-terminated) */
	int   iSaveShort	/* 0=normal save_unit 1=short=only weights */
    );
extern int writeCurrPsomDesc(char* tFileName, char* tRemark);
	/* short interface form to save desc file format */

extern int loadPsomBlock(
	FILE *	fp, /* file pointer to read from */
	unitptr uPsom,  	/* psom unit to be loaded to */
	int *	piLenRest,  	/* pi where <iLenRest> gets copied to */
	float **pafRest,  	/* paf where <afRest[]> gets copied */
	int 	iSkip2Rest,	/* skip reading definition line */
	int 	iCommentMaxLen,	/* max length of comment copied */
	char* 	tCommentBlock	/* comment block copyied to string */
	    );
extern unitptr uPsomFromBlock(
	FILE *	fp,		/* file pointer to read from */
	int 	iNumWeightsExpected, /* expected number of weights */
	int *	piLenRest,	/* pi where <iLenRest> gets copied to */
	float **pafRest,	/* paf where <afRest[]> gets copied */
	int 	iCommentMaxLen,	/* max length of comment copied */
	char* 	tCommentBlock,	/* comment block copyied to string */
	int *	piError,	/* error flag (!=0), returned */
	unitptr uHost		/* host unit */
	    );
int	psomLoadData(FILE* fp);
 	/* 
 	 *	load data from file pointer
	 *	return number of read weights, negative if error
	 *	(#-comments allowed between vectors)
	 */


int	psomExecIter1(void); 
extern int  psomExecComplete(void);
extern int  psomXoutByYin(void);
extern int  psomXoutByYout(void);
 
unitptr	uPsomFromStr(
	int iNumWeightsExpected,	/* expected number of weights if >0*/
	char * tDef,			/* string for definition */
	int *piError,			/* error flag (!=0), returned */
	unitptr uHost			/* host unit */
	);
/*
    *	create and return psom2 by string definition
    *	format   
	<iXdim> <iMdim> <aiKnotMax[0]> <aiKnotMax[1]>  . <aiKnotMax[iMdim-1]>
	<tPSOMctrl token>*
    *
    *	if  iNumWeightsExpected>0 and mismatch no create
    */



extern int	boundXdim(void);
extern int	unboundXdim(void);

extern int	iXdimGetCurrPsom(void)   ;
extern int	iMdimGetCurrPsom(void)   ;
extern int	iKnotTotNumGetCurrPsom(void);
extern int	*aiKnotsGetCurrPsom(void) ;
extern float	*afDeltaGetCurrPsom(void) ;


float psomMapCoordExcursionMax(void);
  	  /* return Max{|Y(guCurrPsom,1,*)-0.5|}*2   */
float psomMapCoordExcursionNorm(void);
	/*  return Sqrt(Sum{(Y(guCurrPsom,1,*)-0.5)**2})*2  */

/***********************************************************************
 * include file for new psom2 alg
 * OLD-Log: psom2.h,v 
 * Revision 1.2  1994/08/27  15:20:20  walter
 * before reconfig
 *
 * 10.5.94  walter
 *
 ***********************************************************************/
#define LAGRANGEMAX PSOMMAXDIM

/* iMode Bit defines:
    If ( .iMode & PSOMAXISSRCINTERN ) current s value is taken from intern psA->dS else from **psA->ppfS.
*/
#define PSOMAXISSRCINTERN	0x1	/* s-value soucre is struct value, already set */
#define PSOMAXISCLEAROFFKNOTS	0x2	/* blank non-OnSet knots */
#define PSOMAXISCLEAROFFSRUCT	0x4	/* blank non-OnSet struct values */
#define PSOMAXISONLY_L		0x800	/* compute only pdL[] */
#define PSOMAXISVERBOSE_0	0x10	/* verbose dPsomAxisModSet() */
#define PSOMAXISnoEXITonERROR	0x100	/* will set global in dPsomAxisModSet() */ 


extern int iPsomAxisArrComp_BD2( 
	float	*pfBa,		/* Ba(s) */
	float	*pfDBaDs,	/* first derivatives */
	float	*pfD2BaDsDs, 	/* second derivatives */
	int	*pIdx,		/* index vector specifying knot */
  long unsigned	*plDoSet,	/* defines set of knots by bit value */
	int	iMode		/* mode */
    );
extern int iPsomAxisArrComp_BD2_fast( 
	float	*pfBa,		/* Ba(s) */
	float	*pfDBaDs,	/* first derivatives */
	float	*pfD2BaDsDs, 	/* second derivatives */
	int	*pIdx 		/* index vector specifying knot */
    );
	
extern int psomAxisArrComp_BaVect( float *pfBa, long unsigned *plDoSet, int iMode);

extern void	setCurrPsom(unitptr u);
extern float	psomUniScale(void);
extern void	aiKnotCnt_by_a(int iA);
extern void	psomGetOnSetByInput( void );
extern void	psomGetOnSetByBestMatch(int iBestA);
extern void	psomY0DiffByBa( void );
extern int iArrOffsetFd(double *pdX, float fX, int iN, int iNmax, int iLMR);




/* 
 *	EXTRA MODE STUFF
 *
 */
#define PSOM_XMODE_UNISCALE    	1	/* no rescale of Xdim */
#define PSOM_XMODE_RESCALE_EQUI	0	/* rescale to equistrong iXdim */
#define PSOM_XMODE_SCALE_MASK	1	/* bit mask */



#define PAX_SETVAL_LINEAR		0	
#define PAX_SETVAL_CHEBY		1	
#define PAX_SETVAL_HYPODEFAULT		2	
#define PAX_SETVAL_F_ARRAY		0x010	
#define PAX_SETVAL_F_1VECTOR		0x110  /* use ppf[0][*] for all axis */


/* 
 *	CENTERED STUFF
 *
 *	defines for axisCheckCentered()
 *	PsomAxis_Centered_* 
 */
#define PSOM_HYPO_MAXSHIFTCNT_DEFAULT	2	/* for create call */

#define	PAX_C_OK		0x0	/* ok */
#define	PAX_C_UP		0x2	/* should shift up */
#define	PAX_C_DOWN		0x1	/* should shift down */
#define	PAX_C_CANT		0x100	/* can't shift, set border reached */
#define	PAX_Cret_2MANY_SHIFT 	0x20000 /* no shift any more (return value) */
#define	PAX_Cret_DID_SHIFT 	0x10000 /* did shift  (return value) */

	/*
	 *  these value define the behavior of axisCheckSetCentered()
	 *  for odd knot sets
	 *	set in global (int) giPsomHypoAxisCenteredBehave
	 */	 
enum _PsomHypoAxisCenteredBehave
  {	PAX_C_OFF,			/* turned off */
	PAX_CODD_WANT_UPPER	,	/* want upper interval, if iLen odd */
	PAX_CODD_WANT_LOWER	,	/* want lower interval */
	PAX_CODD_NOCARE 		/* don't care */
    } ;
extern enum _PsomHypoAxisCenteredBehave giPsomHypoAxisCenteredBehave;
#define TEXT_PsomHypoAxisCenteredBehave	\
	{"OFF", "ODD_WANT_LOWER", "ODD_WANT_UPPER", "ODD_NOCARE"}

extern char * tCtrlPsom2(unitptr u, char* tCtrl);
extern int setCurrPsom_test(unitptr);


 

/* #ifndef NST_ERRORCHECKING_OFF 
   #define psom2		error_spotting(psom2)
   #endif
   */
#ifndef __aMACROS
#define __aMACROS
			/* copy and zero for (float*) arrays  */
#define afCOPY( TO, FROM, LENofFloatArray ) \
     memmove( (char*)(TO), (char*)(FROM), (LENofFloatArray)*sizeof(float))
     /* This is the sprintf() strcpy() convention <To,From,..> */
     
#define afZERO( TO, LENofFloatArray )  \
     memset( (char*)(TO), 0, (LENofFloatArray)*sizeof(float))
     
     /* copy and zero for (double*) arrays  */
#define adCOPY( TO, FROM, LENofDoubleArray ) \
     memmove( (char*)(TO), (char*)(FROM), (LENofDoubleArray)*sizeof(double))
     /* This is the sprintf() strcpy() convention <To,From,..> */
     
#define adZERO( TO, LENofDoubleArray )  \
     memset( (char*)(TO), 0, (LENofDoubleArray)*sizeof(double))
     
#define aiCOPY( TO, FROM, LENofArray ) \
     memmove( (char*)(TO), (char*)(FROM), (LENofArray)*sizeof(int))
#define alCOPY( TO, FROM, LENofArray ) \
     memmove( (char*)(TO), (char*)(FROM), (LENofArray)*sizeof(long))
     
#endif	/* __afMACROS */


#endif  /* _NST_PSOM_H */
