/***************************************************************************
 * 
 * Utility routines for SVD vector handling within prog-unit
 * compiles into "???" package
 *
 * JW 1-2000
 * $Id$
 * Log at file end
 ***************************************************************************/

#ifndef _VECTOR_SVD
#define _VECTOR_SVD

#ifdef SVD_DEFINE_CLASS_METHODS_WITH_NAME
char *SVD_DEFINE_CLASS_METHODS_WITH_NAME[]= {
  "jwCS_:",
  "svd<;;>",			/* constructor (m,n), m>=n */
  "FREE_svd<>",
  "COPY_svd<>",
  "newA<*>",			/* set new A[0..n*m-1], do SVD+zeroing */
  "x_by_b<**>",			/* solve A x=b, return x  */
  "setZeroClipRatio<1>",	/* set, default 1e-6 */
  "getW<*>",			/* get diag of W matrix */
  "getU<*.>",			/* get col i of U, return W_ii [m] */
  "getV<*.>",			/* get col i of V, return V_ii [n] */
  "cntZero<0>",			/* number of zeroed W elements */
  "getDet<0>",			/* det of W = trace (after zeroing) */
  "getDetOrig<0>",		/* det of W (before zeroing */
  "getWmax<0>",			/* max w value */
  "DESCRIBE_svd<>",
  "jwCS_SVD_:",  "print<1>",	/* print to stdout */
  (char*)NULL
};
#endif /* SVD_DEFINE_CLASS_METHODS_WITH_NAME */


struct _svd_t {	
  int 	m;			/* matrix dimension m>=n */
  int	n;			/* matrix dimension */
  int   iMcall;			/* m orig, if m>=n then iMcall=m else set m:=n>iMcall */
  double  *adA0mn;		/* C-like A Matrix [0..m*n-1] */
  double **ppdA;		/* decomposed matrix a[1..m][1..n] */
  double  *adW0n;		/* diagonal Matrix as w0[0..n-1] C-like */
  double  *pdW;			/* diagonal Matrix as w[1..n] */
  double  *adV0nn;		/* Matrix as v0[0..n*n-1] C-like */
  double **ppdV;		/* Matrix v[1..n][1..n] */
  double   dDet;		/* determinate of uncleaned svd */
  double   dDetRange;		/* determinate of zeroed svd */
  int	   iNullity;		/* number of zeroed w[] */
  double   dWmax;		/* largest w[] value (-1.0 not valid)*/
  double   dZeroClipRatio;	/* zeroing factor  */
  double  *adTmpN;		/* temp vector [0..n] */
  double  *adB0m;		/* B util vector [0..m] */
};
typedef struct _svd_t svd_t;

extern svd_t *SVD_new(int m, int n);  /* new */
extern void   jwCS_FREE_svd(svd_t *psOld); /*free */

extern float jwCS_SVD_print(float, svd_t* ps);  /* svd.print(1)K */

extern int SVD_decomposeAndZero(svd_t* ps);  /* do SVD and zero as required */

extern void SVD_x_by_b(double *pdX0result, svd_t *psA , double *pdB0);  /*
    Solves A.X=B for a vector pdX0result[0..n-1] and pdB0[0..m-1] where matrix A
    is spectified by the arrays u,w,v  as an internal result from calling
    dsvdcmpByStruct() before.
    This routine can be called repeated times.  pdX0result==pdB0 is ok.
  */


extern float jwCS_setZeroClipRatio(float fRatio, svd_t* ps);
/* set ratio new if arg in [0,1], else promt current value;
   return previous value */
extern float jwCS_getDet(svd_t* ps);  /*  return det of W = product of diagonal (after zeroing) */
extern float jwCS_getDetOrig(svd_t* ps);  /*  return det of W = product of diagonal (before zeroing) */
extern float jwCS_cntZero(svd_t* ps);  /*  return number of zeroed W elements  */
extern float jwCS_getWmax(svd_t* ps);  /*  return max W element  */
extern float jwCS_getW(int* iDim, float**ppfArg, svd_t* ps);  /* svd.getW(W) := get diag of W matrix */
extern float jwCS_getU(int* iDim, float**ppfArg, svd_t* ps);  /* svd.getU(pfU,c) := get col .c of U matrix and fill pfU[]; */
extern float jwCS_getV(int* iDim, float**ppfArg, svd_t* ps);   /* svd.getV(pfV,c) := get col .c of V matrix and fill pfV[n]; */

extern double *SVD_getRef_A(svd_t *ps);  /* return storage reference to orig A[m*n]*/
extern double *SVD_getRef_b(svd_t *ps);  /* return storage reference to b[m] */


#endif  /* include */

/*
 * $Log$
 */

