/**********************************************************************

  Example of three coupled classes, implemented in plain C:

  class picture:  objects to represent simple pictures 
                  of lines and points
  class line:     a line object that can be added to a picture
  class point:    a point object that can be added to a picture

  The latter two have no member functions and, therefore, just
  implement a new data type that can be used by the member function(s)
  of the first class.

  The example doesn't really care to implement the suggested
  operations. It merely contains the necessary interfaces and some
  code to print info about called functions and their arguments.

***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

   char *drawing_c[] = {
      "__:",  "picture(void)",
              "~picture()",
      "_0_:", "void add(class picture)",
      "_1_:", "void add(int, class line)",
      "_2_:", "void add(class point ...)",
      "_3_:", "void add(class point, int* ...)",
   NULL};

   char *line_c[]  = { 
       "__:", "line(float,float,float,float)", 
   NULL };

   char *point_c[] = { 
       "__:", "point(float,float)",
   NULL};

   char **CLASSES[] = {drawing_c, line_c, point_c, NULL};


/*************************************************************************
  C data types to implement class objects point, line and drawing:
*************************************************************************/

typedef struct point_{ float x,y; } point_t;

typedef struct line_{float x1,y1,x2,y2; } line_t;

typedef struct picture_{ 
   int iNumPoints; point_t **ppPoints; 
   int iNumLines;  line_t **ppLines; 
} picture_t;
 
/*************************************************************************
  wrappers for class constructors:  
*************************************************************************/
picture_t *__picture(void) {
   picture_t *pData = (picture_t*) calloc(1,sizeof(picture_t));
   return pData;
}

point_t *__point(int *piDim, void **ppvArg) {
   float *pfX = ppvArg[0], *pfY = ppvArg[1];
   point_t *pData = (point_t*) calloc(1,sizeof(point_t));
   pData->x = *pfX; pData->y = *pfY;
   return pData;
}

line_t *__line(int *piDim, void **ppvArg) {
   float *pfX1 = ppvArg[0], *pfY1 = ppvArg[1];
   float *pfX2 = ppvArg[2], *pfY2 = ppvArg[3];
   line_t *pData = (line_t*) calloc(1,sizeof(line_t));
   pData->x1 = *pfX1; pData->y1 = *pfY1;
   pData->x2 = *pfX2; pData->y2 = *pfY2;
   return pData;
}

/*************************************************************************
  wrapper for class destructor:
*************************************************************************/

void __FREE_picture(picture_t *pData) {
   /* implementation must permit NULL argument! */
   if (pData) {
      free(pData->ppPoints);
      free(pData->ppLines);
      free(pData);
   }
}

/*************************************************************************
  overloaded class methods add():
*************************************************************************/

void _0_add(int *piDim, void **ppvArg, picture_t *pData) {
   picture_t *pArg0 = (picture_t*) ppvArg[0];
   printf("called drawing.add(class drawing = %p)\n",pArg0);
   /* ... code to add drawing pArg0 to picture pData ... */
}

void _1_add(int *piDim, void **ppvArg, picture_t *pData) {
   int *piArg0   = (int*) ppvArg[0];
   line_t *pArg1 = (line_t*) ppvArg[1];
   if (!pArg1) {  
      printf("draw_ing.add(int,line_t): 2nd arg is not instantiated!\n"); 
      return;
   }
   printf("called drawing.add(int = %d line = (%g %g -> %g %g))\n",
           *piArg0,pArg1->x1,pArg1->y1,pArg1->x2,pArg1->y2);
   /* ... code to add line pArg1 to picture pData ... */
}

void _2_add(int iNumArg, int *piDim, void **ppvArg, picture_t *pData) {
   int i;
   point_t *pArg;
   printf("called drawing.add(class point ... )\n");
   printf("arguments are:\n");
   for (i=0; i<iNumArg; i++) {
      pArg = (point_t*) ppvArg[i];
      printf("point #%d:  x=%g y=%g\n",i, pArg->x, pArg->y);
   }
   /* ... code to add points to picture pData ... */
}

void _3_add(int iNumArg, int *piDim, void **ppvArg, picture_t *pData) {
   printf("called drawing.add(class point)\n");
}
